/* global _ */
/* global angular */

angular.module('services', [])
    .factory('$localStorage', ['$window',
        function ($window) {
            'use strict';

            return {
                name: "pocketcards",

                store: (function () {
                    return $window.localStorage.getItem("pocketcards");
                }()),

                records: (function () {
                    var store = $window.localStorage.getItem("pocketcards");
                    return (store && store.split(",")) || [];
                }()),

                save: function () {
                    $window.localStorage.setItem(this.name, this.records.join(","));
                },

                create: function (model) {
                    if (!model.id) {
                        throw new Error("Model should have a unique id!");
                    }
                    $window.localStorage.setItem(this.name + "-" + model.id, JSON.stringify(model));
                    this.records.push(model.id.toString());
                    this.save();
                    return this.find(model);
                    // $window.localStorage[key] = JSON.stringify(value);
                },

                // Update a model by replacing its copy in `this.data`.
                update: function (model) {
                    $window.localStorage.setItem(this.name + "-" + model.id, JSON.stringify(model));
                    if (!_.include(this.records, model.id.toString())) {
                        this.records.push(model.id.toString());
                    }
                    this.save();
                    return this.find(model);
                },

                // Retrieve a model from `this.data` by id.
                find: function (model) {
                    return this.jsonData($window.localStorage.getItem(this.name + "-" + model.id));
                },

                // Return the array of all models currently in storage.
                findAll: function () {

                    // Lodash removed _#chain in v1.0.0-rc.1
                    return (_.chain || _)(this.records)
                        .map(function (id) {
                            return this.jsonData($window.localStorage.getItem(this.name + "-" + id));
                        }, this)
                        .compact()
                        .value();
                },

                // Delete a model from `this.data`, returning it.
                destroy: function (model) {
                    // if (model.isNew())
                    //     return false
                    $window.localStorage.removeItem(this.name + "-" + model.id);
                    this.records = _.reject(this.records, function (id) {
                        return id === model.id.toString();
                    });
                    this.save();
                    return model;
                },

                // Fix for "illegal access" error on Android when JSON.parse is passed null
                jsonData: function (data) {
                    return data && JSON.parse(data);
                }
            };
        }
    ])
    .factory("$webService", function ($http, $rootScope, $q) {
        'use strict';

        // TODO: Maybe cache pocket cards????
        // See the follow for explaination on error code 0
        // http://stackoverflow.com/questions/17929804/http-status-0-from-angularjs-get-for-json
        var cards = [];

        // Display annoying alert message once.
        var _alert = _.once(function (msg) {
            if (window.ionic.Platform.isWebView()) {
                navigator.notification.alert(msg, function () {});
            } else {
                alert(msg);
            }
        });

        function success(response) {
            if (response.config.cache === false) {
                $rootScope.isOnline = true;
            }

            cards = response.data;
            return cards;
        }

        function fail(response) {
            if (response.config.cache === false) {
                $rootScope.isOnline = false;
            }

            _alert("Error! Unable to retrieve pocketcards at this time. Try again later.", function () {});
            return $q.reject(response);
        }

        // TODO: Change localhost to actual name of host where this server will be hosted. Make this
        // configurable.
        return {
            getPocketCards: function (mode) {
                var isCached = mode || false;
                return $http.get($rootScope.myHost + "/pocketcardv1/user/" + $rootScope.device_hash + "/cards", {
                    cache: isCached
                }).then(success, fail);
            },

            getPocketCard: function (id, mode) {
                var isCached = mode || false;
                return $http.get($rootScope.myHost + "/pocketcardv1/user/" + $rootScope.device_hash + "/cards/" + id, {
                    cache: isCached
                }).then(success, fail);
            },
            getPocketCardContent: function (id, mode) {
                var isCached = mode || false;
                return $http.get($rootScope.myHost + "/pocketcardv1/user/" + $rootScope.device_hash + "/cards/" + id + "/contents", {
                    cache: isCached
                }).then(success, fail);
            }
        };
    });