var express = require('express');
var path = require('path');
var logger = require('morgan');
var log4js = require('log4js');
var package = require('./package').config;

var admin = require('./routes/admin');

var app = express();

var log = log4js.getLogger('app');

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'jade');

app.use(logger('dev'));

app.all('*', function (req, res, next) {
	log.debug("originalUrl:" + req.originalUrl + " url:" + req.url + " 'host':" + req.get("host"));
	next();
});

app.use(express.static(path.join(__dirname, 'public')));

app.use('/', admin);

// catch 404 and forward to error handler
app.use(function(req, res, next) {
	var err = new Error('Not Found');
	err.status = 404;
	log.debug('forwarding a 404');
	next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
	app.use(function(err, req, res, next) {
		res.status(err.status || 500);
		res.render('error', {
			message: err.message,
			error: err
		});
		log.error('dev err',err);
	});
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
	res.status(err.status || 500);
	res.render('error', {
		message: err.message,
		error: {}
	});
	log.error('prod err', err);
});

app.set('port', process.env.npm_config_processPort || package.processPort);

if (package.https) {
	var https = require('https');
	var fs = require('fs');
	var httpsOptions = {
		key: fs.readFileSync(package.keyFileName),
		cert: fs.readFileSync(package.certFileName)
	};
	var server = https.createServer(httpsOptions, app).listen(app.get('port'), function() {
		log.info('Express server listening on port ' + server.address().port);
	});
}
else {
	var server = app.listen(app.get('port'), function() {
		log.info('Express server listening on port ' + server.address().port + ' ' + process.pid);
	});
}
