//Dummy dao to use for testing: NOT TO BE USED IN PRODUCTION
//
var log4js = require('log4js');
var dao_utils = require('./dao_utils.js');
var log = log4js.getLogger('AdminDao_Dummy');
var _ = require('underscore');

var DummyDao = function () {
	this.db = require('./dummy_db.js');
};

DummyDao.prototype.getCurrentUserBySysid = function (sysid, finished) {
	var resultSet = {};
	var now = new Date();

	var users = getUsersSync(this.db);

	var results = null;

	for (var i = users.length - 1; i >= 0; i--) {
		var u_rec = users[i];
		if (sysid==u_rec.activeDirId) {
			if (!results) {
				results = [];
			}
			results.push(u_rec);
		}
	};

	finished(null, results);
};

DummyDao.prototype.getCurrentUserIdsBySysid = function (id, callback) {
	this.getCurrentUserBySysid(id, function (err, users) {
		var results = null;
		if (users && users.length) {
			results = [];
			for (var i = users.length - 1; i >= 0; i--) {
				results.push(users[i].id);
			};
		}
		callback(err, results);
	});
};

DummyDao.prototype.recordAction = function (data, callback) {
	this.db.audits.push(data);
	callback();
};

DummyDao.prototype.saveNewUser = function (user, data, callback) {
	var newUserId = 0;
	for (var i=0; i<this.db.admin_users.length; i++) {
		if (this.db.admin_users[i].id > newUserId) {
			newUserId = this.db.admin_users[i].id;
		}
	}
	if (!newUserId) {
		return callback(new Error('no new user ID found'));
	}
	newUserId++;
	var newEntry = dao_utils.getDbUserEntry(data);
	newEntry.id = newUserId;
	this.db.admin_users.push(newEntry);
	callback(null, newUserId);
};

DummyDao.prototype.updateUser = function (data, callback) {
	if (!this.userExists(data.id)) {
		var msg = 'updateUser user.id unknown: ' + data.id;
		log.error(msg);
		return callback(new Error(msg));
	}

	this.db.admin_users.push(dao_utils.getDbUserEntry(data));
	callback();
};

DummyDao.prototype.userExists = function (id) {
	if (id) {
		for (var i=0; i<this.db.admin_users.length; i++) {
			if (id==this.db.admin_users[i].id) {
				return true;
			}
		}
	}
	return false;
};

DummyDao.prototype.getCurrentUserById = function (userId, callback) {
	callback(null, getCurrentUserByIdSync(userId, this.db));
};

function getCurrentUserByIdSync(userId, db) {
	var result = null;
	var now = new Date();
	for (var i=0; i<db.admin_users.length; i++) {
		var user = db.admin_users[i];
		if (user && user.id && userId==user.id && user.effective_date <= now) {
			if (null==result) {
				result = user;
			}
			else if (user.effective_date > result.effective_date) {
				result = user;
			}
		}
	}
	return dao_utils.getNormalUserEntry(result);
}

DummyDao.prototype.saveNewCard = function (user, newCard, callback) {
	var nextIdNum = getHighestId(this.db.cards) + 1;
	newCard.id = nextIdNum;
	this.db.cards.push(newCard);
	callback(null, nextIdNum);
};

DummyDao.prototype.getCurrentCard = function (cardId, callback) {
	var result = null;
	var now = new Date();
	var lastDate = new Date('1900-01-01 00:00:00');
	for (var i=0;i<this.db.cards.length;i++) {
		var card = this.db.cards[i];
		if (cardId===card.id && card.effectiveDate && card.effectiveDate>lastDate && card.effectiveDate<=now) {
			lastDate = card.effectiveDate;
			result = card;
		}
	}
	callback(null, result);
}

DummyDao.prototype.getCurrentCards = function (userId, callback) {
	var results = [];
	if (this.db.cards && this.db.cards.length) {
		var now = new Date();
		for (var i=0;i<this.db.cards.length;i++) {
			if (userId && this.db.cards[i].ownerId && this.db.cards[i].ownerId!=userId) {
				continue;
			}
			var cardInfo = getCard(this.db.cards[i].id, results);
			if (!cardInfo.card) {
				results.push(this.db.cards[i]);
			}
			else {
				if (this.db.cards[i].effectiveDate>cardInfo.card.effectiveDate && this.db.cards[i].effectiveDate<now) {
					results[cardInfo.index] = this.db.cards[i];
				}
			}
		}
	}
	callback(null, results);
};

DummyDao.prototype.updateCard = function (card, callback) {
	this.db.cards.push(card);
	callback();
};

DummyDao.prototype.getUsers = function (callback) {
	callback(null, getUsersSync(this.db));
};

function getUsersSync (db) {
	var now = new Date();
	var set = {};
	for (var i = db.admin_users.length - 1; i >= 0; i--) {
		if (db.admin_users[i].effective_date<=now) {
			if (set[db.admin_users[i].id]) {
				if (db.admin_users[i].effective_date > set[db.admin_users[i].id].effective_date) {
					set[db.admin_users[i].id] = db.admin_users[i];
				}
			}
			else {
				set[db.admin_users[i].id] = db.admin_users[i];
			}
		}
	};
	var list = [];
	for (var id in set) {list.push(dao_utils.getNormalUserEntry(set[id]))};
	return list;
}

DummyDao.prototype.getCardFile = function (cardId, callback) {
	this.getCurrentCard(cardId, function (err, result) {
		if (err) {
			return callback(err);
		}
		if (result) {
			callback(null, result.file);
		}
		else {
			callback();
		}
	});
};

DummyDao.prototype.getCardFileInstance = function (cardId, instanceId, callback) {
	var cards = this.db.cards;
	var result = null;
	for (var i = cards.length - 1; i >= 0; i--) {
		if (cardId===cards[i].id && instanceId===getInstanceId(cards[i])) {
			result = cards[i];
		}
	};
	if (result && result.file) {
		callback(null, result.file);
	}
	else {
		callback();
	}
}

DummyDao.prototype.getCardHistory = function (cardId, callback) {
	if (this.db.cards && this.db.cards.length) {
		var ary = [];
		for (var i = this.db.cards.length - 1; i >= 0; i--) {
			if (this.db.cards[i].id===cardId) {
				ary.push(this.db.cards[i]);
			}
		};
		ary.sort(function (c1, c2) {
			if (c1 && c2) {
				return c1.effectiveDate.getTime() - c2.effectiveDate.getTime();
			}
			if (c1) {
				return 1;
			}
			if (c2) {
				return -1;
			}
			return 0;
		});
		ary.forEach(function (item) {
			item.instanceId = item.effectiveDate.getTime().toString();
		});
		callback(null, ary);
	}
	else {
		callback();
	}
};

DummyDao.prototype.getCardAuditHistory = function (cardId, callback) {
	if (this.db.audits && this.db.audits.length) {
		var ary = [];
		for (var i = this.db.audits.length - 1; i >= 0; i--) {
			if (cardId===this.db.audits[i].itemCardId) {
				ary.push(this.db.audits[i]);
			}
		};
		callback(null, ary);
	}
	else {
		callback();
	}
};

DummyDao.prototype.setPersonNames = function (personIdNameOb, callback) {
	for (var id in personIdNameOb) {
		var user = getCurrentUserByIdSync(id, this.db);
		if (user && user.lastname && user.firstname) {
			personIdNameOb[id] = user.firstname + ' ' + user.lastname;
		}
	}
	callback();
};

DummyDao.prototype.getAllCurrentUserNames = function (callback) {
	var list = getUsersSync(this.db);
	if (list && list.length) {
		for (var i = list.length - 1; i >= 0; i--) {
			list[i] = {id: list[i].id, firstname: list[i].firstname, lastname: list[i].lastname};
		};
	}
	callback(null, list);
};

DummyDao.prototype.getMetaMetrics = function (fromDt, toDt, callback) {
	callback(null, _.filter(this.db.metrics, function (metric) {
		if (metric.timestamp>=fromDt && metric.timestamp<=toDt) {
			var matches = metric.request.match(/cards(\?.*)?$/);
			if (matches) {
				return true;
			}
		}
		return false;
	}));
};

DummyDao.prototype.getMetaMetricsIterator = function (fromDt, toDt, callback) {
	callback(null, getMetricsIterator(this.db, fromDt, toDt));
};

DummyDao.prototype.getCardMetricsIterator = function (cardId, fromDt, toDt, callback) {
	callback(null, new CardMetricsIterator(this.db, cardId, fromDt, toDt));
};

function getCard(id, cards) {
	if (cards && cards.length) {
		for (var i=0;i<cards.length;i++) {
			if (id==cards[i].id) {
				return {index:i, card: cards[i]};
			}
		}
	}
	return {};
}

function getHighestId(ary) {
	var result = 0;
	if (ary && ary.length) {
		for(var i=0;i<ary.length;i++) {
			if (ary[i] && ary[i].id && ary[i].id>result) {
				result = ary[i].id
			}
		}
	}
	return result;
}

function getInstanceId(item) {
	if (item) {
		if (item.instanceId) {
			return item.instanceId;
		}
		else {
			return item.id.toString() + '_' + item.effectiveDate.getTime().toString();
		}
	}
	return null;
}

function getMetricsIterator(db, fromDate, toDate) {
	return {
		fromDate: fromDate,
		toDate: toDate,
		metrics : db.metrics.slice(0),
		pos : 0,
		getNext: function (callback) {
			if (this.pos>=this.metrics.length) {
				return callback();
			}
			var result = null;
			do {
				var item = this.metrics[this.pos];
				this.pos++;
				if (item.timestamp>=this.fromDate && item.timestamp<=this.toDate && item.request.match(/cards(\?.*)?$/)) {
					result = item;
					break;
				}
			} while (this.pos<this.metrics.length);
			callback(null, result);
		},
		done: function () {
			this.metrics = [];
		}
	};
}

function CardMetricsIterator (db, cardId, fromDt, toDt) {
	if (cardId) {
		this.re = new RegExp('cards/' + cardId + '/contents');
	}
	else {
		this.re = new RegExp('cards/.+/contents');
	}
	this.db = db;
	this.pos = 0;
	this.fromDt = fromDt;
	this.toDt = toDt;

	this.getNext = function (callback) {
		var result = null;
		while (null==result && this.pos<this.db.metrics.length) {
			var item = this.db.metrics[this.pos];
			if (this.passesItemCheck(item) && this.passesDateCheck(item)) {
				result = item;
			}
			this.pos++;
		}
		callback(null, result);
	};

	this.passesItemCheck = function (item) {
		if (item) {
			if (!this.re.exec(item.request)) {
				return false;
			}
			return true;
		}
		return false;
	};

	this.passesDateCheck = function (item) {
		if (item) {
			if (this.fromDt) {
				if (item.timestamp.getTime()<this.fromDt.getTime() || item.timestamp.getTime()>this.toDt.getTime()) {
					return false;
				}
			}
			return true;
		}
		return false;
	};

	this.done = function () {
		this.pos = this.db.metrics.length;
	};

	return this;
}

module.exports = DummyDao;
