//This is a facade class that adds data and method checking to any DAO used in the system.
// This code exists so it doesn't have to be rewritten for every DAO written for the system.
//
var log4js = require('log4js');

var log = log4js.getLogger('IAdminDao');

var Daoi = function (aDao) {
	this.dao = aDao;
};

Daoi.prototype.recordAction = function (data, callback) {
	this.assertDao('recordAction');
	if (!data) {
		return callback(new Error('data object empty'));
	}
	if (!assertAuditDataIsValid(data)) {
		var message = 'recordAction missing data: only got ' + JSON.stringify(Object.keys(data));
		log.error(message);
		return callback(new Error(message));
	}

	this.dao.recordAction(data, callback);
};

Daoi.prototype.saveNewUser = function (user, data, callback) {
	this.assertDao('saveNewUser');
	this.assertUser(user);

	if (!data) {
		return callback(new Error('data object empty'));
	}
	if (!this.assertUserData(data)) {
		var message = 'saveNewUser bad input parameters; only had ' + JSON.stringify(Object.keys(data));
		log.error(message);
		return callback(new Error(message));
	}

	this.dao.saveNewUser(user, data, callback);
};

Daoi.prototype.updateUser = function (data, callback) {
	this.assertDao('updateUser');

	if (!data) {
		var msg = 'updateUser data empty';
		log.error(msg);
		return callback(new Error(msg));
	}

	if (!this.assertUserData(data) || !data.id) {
		var msg = 'updateUser bad input parameters; only had ' + JSON.stringify(Object.keys(data));
		log.error(msg);
		return callback(new Error(msg));
	}

	this.dao.updateUser(data, callback);
};

Daoi.prototype.saveNewCard = function (user, data, callback) {
	this.assertDao('saveNewCard');
	this.assertUser(user);
	var cardAssertFailure = assertCardData(data);
	if (cardAssertFailure) {
		return callback(new Error(cardAssertFailure));
	}
	this.dao.saveNewCard(user, data, callback);
};

Daoi.prototype.updateCard = function (card, callback) {
	this.assertDao('updateCard');
	var cardAssertFailure = assertCardData(card);
	if (cardAssertFailure) {
		return callback(new Error(cardAssertFailure));
	}
	this.dao.updateCard(card, callback);
}

Daoi.prototype.assertUserData = function (user) {
	var result = (user && 
		user.userType && 
		user.lastname && 
		user.firstname && 
		user.activeDirId && 
		user.status && 
		user.effectiveDate &&
		('[object Date]' == Object.prototype.toString.call(user.effectiveDate)));
	return result;
};

Daoi.prototype.getCurrentUserById = function (userId, callback) {
	this.assertDao('getCurrentUserById');
	if (!userId) {
		return callback(new Error('no userId given'));
	}
	this.dao.getCurrentUserById(userId, callback);
};

Daoi.prototype.getCurrentCard = function (cardId, callback) {
	this.assertDao('getCurrentCard');
	if (!cardId) {
		return callback(new Error('no cardId given'));
	}
	this.dao.getCurrentCard(cardId, callback);
};

Daoi.prototype.getCurrentCards = function (userId, callback) {
	if (null==callback) {
		callback = userId;
		userId = null;
	}
	this.dao.getCurrentCards(userId, callback);
}

Daoi.prototype.getCurrentUserBySysid = function (sysid, callback) {
	this.assertDao('getCurrentUserBySysid');
	if (!sysid) {
		return callback(new Error('no sysid'));
	}
	this.dao.getCurrentUserBySysid(sysid, callback);
};

Daoi.prototype.getCurrentUserIdsBySysid = function (sysid, callback) {
	this.assertDao('getCurrentUserIdsBySysid');
	if (!sysid) {
		return callback(new Error('no sysid'));
	}
	this.dao.getCurrentUserIdsBySysid(sysid, callback);
};

Daoi.prototype.getUsers = function (callback) {
	this.assertDao('getUsers');
	this.dao.getUsers(callback);
};

Daoi.prototype.getCardFile = function (cardId, callback) {
	this.assertDao('getCardFile');
	if (!cardId) {
		return callback(new Error('no cardId'));
	}
	this.dao.getCardFile(cardId, callback);
};

Daoi.prototype.getCardFileInstance = function (cardId, instanceId, callback) {
	this.assertDao('getCardFileInstance');
	if (!cardId) {
		return callback(new Error('no cardId'));
	}
	if (!instanceId) {
		return callback(new Error('no instanceId'));
	}
	this.dao.getCardFileInstance(cardId, instanceId, callback);
};

Daoi.prototype.getCardHistory = function (cardId, callback) {
	this.assertDao('getCardHistory');
	if (!cardId) {
		return callback(new Error('no cardId'));
	}
	var id = parseInt(cardId);
	if ('NaN'===id.toString()) {
		return callback(new Error('non-numeric card ID'));
	}
	this.dao.getCardHistory(id, function (err, results) {
		if (results && results.length) {
			results.forEach(function (item) {
				if (!item.instanceId) {
					return callback(new Error('some results do not have an instanceId'));
				}
			});
		}
		callback(err, results);
	});
};

Daoi.prototype.getCardAuditHistory = function (cardId, callback) {
	this.assertDao('getCardAuditHistory');
	if (!cardId) {
		return callback(new Error('no cardId'));
	}
	this.dao.getCardAuditHistory(cardId, callback);
};

Daoi.prototype.setPersonNames = function (personIdNameOb, callback) {
	this.assertDao('setPersonNames');
	if (personIdNameOb) {
		this.dao.setPersonNames(personIdNameOb, callback);
	}
	else {
		callback();
	}
};

Daoi.prototype.getAllCurrentUserNames = function (callback) {
	this.assertDao('getAllCurrentUserNames');
	this.dao.getAllCurrentUserNames(callback);
};

Daoi.prototype.getMetaMetricsIterator = function (fromDt, toDt, callback) {
	this.assertDao('getMetaMetricsIterator');
	if (fromDt && toDt) {
		if (isValidDate(fromDt) && isValidDate(toDt)) {
			this.dao.getMetaMetricsIterator(fromDt, toDt, function (err, iter) {
				if (iter && (!iter.getNext || !iter.done)) {
					callback(new Error('iterator missing functions'));
				}
				else {
					callback(err,iter);
				}
			});
		}
		else {
			callback(new Error('date(s) format problem'));
		}
	}
	else {
		callback(new Error('date(s) were empty'));
	}
};

Daoi.prototype.getCardMetricsIterator = function (cardId, fromDt, toDt, callback) {
	this.assertDao('getCardMetricsIterator');
	if (fromDt || toDt) {
		if (!(fromDt && toDt)) {
			return callback(new Error("one of the query dates missing"));
		}
		if (!isValidDate(fromDt)) {
			return callback(new Error("'from' date is invalid"));
		}
		if (!isValidDate(toDt)) {
			return callback(new Error("'to' date is invalid"));
		}
	}
	this.dao.getCardMetricsIterator(cardId, fromDt, toDt, function(err, iter) {
		if (iter && (!iter.getNext || !iter.done)) {
			callback(new Error('iterator missing functions'));
		}
		else {
			callback(err, iter);
		}
	});
};

Daoi.prototype.assertDao = function (aFunc) {
	if (this.dao && this.dao[aFunc]) {}
	else {
		var msg = null;
		if (this.dao) {
			msg = "no dao function found '" + aFunc + "'";
		}
		else {
			msg = 'no dao found';
		}
		log.error(msg);
		throw new Error(msg);
	}
};

Daoi.prototype.assertUser = function (user) {
	if (!user) {
		var msg = 'no user object';
		log.error(msg);
		throw new Error(msg);
	}
	if (!user.id) {
		var msg = 'no user.id';
		log.error(msg);
		throw new Error(msg);
	}
};

function assertCardData (data) {
	if (!data) {
		return 'no card data';
	}
	if (!data.status) {
		return 'no status';
	}
	if (!data.effectiveDate) {
		return 'no effectiveDate';
	}
	return null;
};

function assertAuditDataIsValid(data) {
	if (data && data.userId && data.action && data.datetime) {
		if (data.itemCardId || data.itemUserId) {
			if (data.effectiveDate) {
				return true;
			}
		}
		else {
			return true;
		}
	}
	return false;
}

function isValidDate(aDate) {
	if (aDate && aDate.getTime && 'NaN'!==aDate.getTime().toString()) {
		return true;
	}
	return false;
}

module.exports = Daoi;
