/* global angular */
/* global clientserverCommon */
//adminapp.js
var adminapp = angular.module('adminapp', ['ngAria', 'ngDialog', 'filters']);

adminapp.factory('unloadMessage', function () {
	return function () {
		return "Your changes are NOT saved";
	}
});

adminapp.controller('links', ['$scope', '$timeout', '$window', '$http', 'ngDialog', function ($scope, $timeout, $window, $http, ngDialog) {
	var MINUTE = 60*1000;
	$scope.timeout = 15.5 * MINUTE;
	$scope.timeoutWarning = $scope.timeout - (2.5 * MINUTE);

	$scope.logoff = function () {
		$window.onbeforeunload = null;
		$window.location.href = 'logout';
	};

	$scope.reset = function () {
		ngDialog.close();
		$scope.resetLogoffTimeout();
		$scope.resetWarningTimeout();
		$http.get('ping')
			.success(function (data) {
				//no-op
			}).error($scope.logoff);
	};

	$scope.resetWarningTimeout = function () {
		$timeout(function () {
			ngDialog.open({
				template: 'tDialog',
				className: 'ngdialog-theme-default',
				scope: $scope
			});
		}, $scope.timeoutWarning);
	};

	$scope.resetLogoffTimeout = function () {
		if ($scope.logoffPromise) {
			$timeout.cancel($scope.logoffPromise);
		}
		$scope.logoffPromise = $timeout($scope.logoff, $scope.timeout);
	};

	$scope.resetLogoffTimeout();
	$scope.resetWarningTimeout();

}]);

adminapp.directive('fileInput', function () {
	return {
		restrict: 'E',
		templateUrl: 'fileinput.html',
		replace: true,

		scope: {
			onFileChange: '=onFileChange'
		},

		controller: function ($scope) {
			$scope.cancelClick = function () {
				$scope.removeFileInput();
				$scope.prependFileInput();
				$scope.onFileChange();
			};
			$scope.prependFileInput = function () {
				$scope.fileInputContainer.prepend("<input type='file' id='" + $scope.formId + "' name='" + $scope.formId + "' class='btn' accept='text/html' onchange='angular.element(this).scope().onFileChange(this.value);angular.element(this).scope().$apply()'/>");
			};
			$scope.removeFileInput = function () {
				$scope.fileInputContainer.find('input').remove();
			};
		},

		link: function (scope, ielement, attrs) {
			scope.fileInputContainer = ielement;
			scope.formId = attrs.formId;
			scope.prependFileInput();
		}
	};
});

adminapp.factory("utils", function () {
	return {
		isLeapYear: function (year) {
			if (null===year || isNaN(year)) {
				return false;
			}
			return !(year & 3 || year & 15 && !(year % 25));
		},
		isFileValid: function (fileName) {
			if (fileName && (clientserverCommon.endsWith(fileName, '.htm') || clientserverCommon.endsWith(fileName, '.html'))) {
				return true;
			}
			return false;
		}
	};
});

adminapp.directive('lookupsysid', ['$http', '$q', function ($http, $q) {
	return {
		require: 'ngModel',
		link: function (scope, elm, attrs, ctrl) {
			ctrl.$asyncValidators.lookupsysid = function(mValue, vValue) {
				if (ctrl.$isEmpty(mValue)) {
					return $q.reject('empty');
				}

				if (!clientserverCommon.sysidRegex.test(vValue)) {
					return $q.reject('pattern');
				}

				var def = $q.defer();

				$http.get('isidused?id='+mValue)
					.success(function (data) {
						if (data && 'true'==data.toString()) {
							def.reject('used');
						}
						else {
							def.resolve();
						}
					})
					.error(function () {
						def.reject('error')
					});
				return def.promise;
			};

			ctrl.$validators.pattern = function (mValue, vValue) {
				if (clientserverCommon.sysidRegex.test(vValue)) {
					return true;
				}
				return false;
			};
		}
	};
}]);
