//editcard.js
var adminapp = angular.module('adminapp');

adminapp.controller('editcardController', ['$scope', '$http', '$window', 'unloadMessage', 'utils', function ($scope, $http, win, unloadMessageFunc, utils) {
	win.onbeforeunload = unloadMessageFunc;

	$scope.replaceFile = false;
	$scope.lastFileTryError = false;
	$scope.oldStatus = 'inactive';
	$scope.oldVersion = '';
	$scope.oldVersionComment = '';
	$scope.oldTitle = '';
	$scope.oldOwnerId = 0;
	$scope.replaceOwner = false;
	$scope.error = false;
	$scope.loading = true;
	$scope.statusChangeable = false;
	$scope.oldVersions = [];
	$scope.nameListCalled = false;
	$scope.nameList = [{id:-1, name: 'Loading names...'}];

	$scope.submit = function () {
		win.onbeforeunload = null;
	};

	$scope.replaceOwnerChanged = function () {
		if ($scope.replaceOwner) {
			if (!$scope.nameListCalled) {
				$scope.nameListCalled = true;
				$http.get('allCurrentUsersNames')
					.success(function (data) {
						for (var i = data.length - 1; i >= 0; i--) {
							data[i].name = data[i].firstname + ' ' + data[i].lastname;
						};
						data.sort(function (i1, i2) {
							if (i1.lastname===i2.lastname) {
								if (i1.firstname > i2.firstname) {
									return 1;
								}
								if (i1.firstname < i2.firstname) {
									return -1;
								}
								return 0;
							}
							if (i1.firstname > i2.firstname) {
								return 1;
							}
							if (i1.firstname < i2.firstname) {
								return -1;
							}
							return 0;
						});
						$scope.nameList = data;
					})
					.error(function () {
						$scope.nameList = [{id:-1, name:'Error retrieving names'}];
					});
			}
		}
	}

	$scope.setFile = function (file) {
		$scope.newFile = file;
		$scope.lastFileTryError = false;
		if (file) {
			if (utils.isFileValid(file)) {
				if ($scope.card.file && $scope.card.file.originalname) {
					$scope.card.status = $scope.oldStatus;
				}
			}
			else {
				$scope.lastFileTryError = true;
			}
		}
		else {
			$scope.card.status = 'inactive';
		}
		$scope.updateStatusChangeable();
	};

	$scope.isVersionUsedBefore = function () {
		if ($scope.card && $scope.card.version && $scope.oldVersions && $scope.oldVersions.length) {
			for (var i=0;i<$scope.oldVersions.length;i++) {
				if ($scope.card.version==$scope.oldVersions[i]) {
					return true;
				}
			}
		}
		return false;
	}

	$scope.replaceChanged = function () {
		if (!$scope.loading && !$scope.error) {
			if ($scope.replaceFile) {
				if ($scope.newFile) {
					if (!utils.isFileValid($scope.newFile)) {
						$scope.lastFileTryError = true;
					}
					$scope.card.status = $scope.oldStatus;
				}
				else {
					$scope.card.status = 'inactive';
				}
			}
			else {
				$scope.lastFileTryError = false;

				if ($scope.card.file && $scope.card.file.originalname) {
					$scope.card.status = $scope.oldStatus;
				}
				else {
					$scope.card.status = 'inactive';
				}
			}
		}
		$scope.updateStatusChangeable();
	};

	$scope.stringArrayToLines = function (ary) {
		if (ary) {
			var s = '';
			for (var i=0;i<ary.length;i++) {
				s += ary[i];
				if (i<ary.length-1) {
					s += '\r\n';
				}
			}
			return s;
		}
		else {
			return '';
		}
	};

	$scope.updateStatusChangeable = function () {
		if (!this.loading && !this.error) {
			if (this.replaceFile) {
				if (this.newFile) {
					this.statusChangeable = true;
				}
				else {
					this.statusChangeable = false;
				}
			}
			else {
				if (this.card.file && this.card.file.originalname) {
					this.statusChangeable = true;
				}
				else {
					this.statusChangeable = false;
				}
			}
		}
	};

	$scope.isVersionStateOkay = function () {
		if ($scope.card && ($scope.card.version || 'active'!=$scope.card.status)) {
			return true;
		}
		return false;
	};

	$scope.isCommentStateOkay = function () {
		if ($scope.card && ($scope.card.versionComment || ('active'!=$scope.oldStatus && 'active'!=$scope.card.status))) {
			return true;
		}
		return false;
	};

	$http.get('cardversions')
		.success(function (data) {
			$scope.oldVersions = data;
		})
		.error(function () {
			$scope.versionLoadError = true;
			//TODO: show this error and disallow user to continue?
		});

	$http.get('carddata')
		.success(function (data) {
			if (data.tags) {
				data.tags = $scope.stringArrayToLines(data.tags);
			}
			if (data.status) {
				$scope.oldStatus = data.status;
			}
			$scope.oldVersion = data.version;
			$scope.oldVersionComment = data.versionComment;
			$scope.oldTitle = data.title;
			data.versionComment = '';
			data.version = '';
			$scope.loading = false;
			$scope.card = data;
			$scope.updateStatusChangeable();
		})
		.error(function () {
			$scope.loading = false;
			$scope.card = null;
			$scope.error = true;
		});

}]);
