//ui_helper.js
var log4js = require('log4js');
var log = log4js.getLogger('ui_helper');
var fs = require('fs');
var clientserverCommon = require('../public/js/clientserverCommon');

//** IAM Variables **
var IAM_DEV_BASE = 'https://logon.dev2.iam.domain';
var IAM_SQA_BASE = 'https://ssologon.sqa.iam.domain';
var IAM_PREP_BASE = 'https://ssologon.preprod.iam.domain';
var IAM_PROD_BASE = 'https://ssologon.iam.domain';
var IAM_MIDDLE_PATH = '/CentralLogin';
var IAM_EXIT_PATH = '/centrallanding.aspx';
var IAM_LOGOUT_PATH = '/loggedout.aspx';
var IAM_IDLE_LOGOUT_PATH = '/idletimeout.aspx';

var ui_helper = {
	setupNewUser: function (aUser, req) {
		aUser.firstname = req.body.fname;
		aUser.lastname = req.body.lname;
		aUser.activeDirId = req.body.sysid;
		aUser.userType = req.body.userType;
		aUser.status = req.body.initStatus;
		aUser.email = req.body.email;
	},

	setupNewUser2: function (aUser, fields) {
		aUser.firstname = this.getFirstArrayItemOrObject(fields.fname);
		aUser.lastname = this.getFirstArrayItemOrObject(fields.lname);
		aUser.activeDirId = this.getFirstArrayItemOrObject(fields.sysid);
		aUser.userType = this.getFirstArrayItemOrObject(fields.userType);
		aUser.status = this.getFirstArrayItemOrObject(fields.initStatus);
		aUser.email = this.getFirstArrayItemOrObject(fields.email);
	},

	getFirstArrayItemOrObject: function (ob) {
		if (ob) {
			if (Array.isArray(ob)) {
				if (ob.length>0) {
					return ob[0];
				}
			}
			else {
				return ob;
			}
		}
		return null;
	},

	getCard: function (req) {
		var card = {};
		card.title = req.body.title;
		card.descr = req.body.descr;
		var tags = req.body.tags.trim();
		if (''==tags) {
			card.tags = [];
		}
		else {
			card.tags = tags.replace('\r','').split('\n');
		}
		for (var i=0;i<card.tags.length;i++) {
			card.tags[i] = card.tags[i].trim();
		}
		card.version = req.body.version;
		card.versionComment = req.body.versionComment;
		if (req.files.file1) {
			card.file = req.files.file1;
		}
		card.createdBy = req.body.createdBy;
		card.status = req.body.status;
		card.replaceFile = req.body.replaceFile;

		if (req.body.replaceOwner) {
			card.replaceOwner = req.body.replaceOwner;
			if (req.body.ownerId) {
				card.ownerId = req.body.ownerId;
			}
		}
		return card;
	},

	getCard2: function (fields, files, callback) {
		var card = {};
		card.title = this.getFirstArrayItemOrObject(fields.title);
		card.descr = this.getFirstArrayItemOrObject(fields.descr);
		var tags = this.getFirstArrayItemOrObject(fields.tags);
		if (tags) {
			tags = tags.toString().trim();
		}
		if (''===tags) {
			card.tags = [];
		}
		else {
			card.tags = tags.replace('\r','').split('\n');
		}
		for (var i=0;i<card.tags.length;i++) {
			card.tags[i] = card.tags[i].trim();
		}
		card.version = this.getFirstArrayItemOrObject(fields.version);
		card.versionComment = this.getFirstArrayItemOrObject(fields.versionComment);
		card.status = this.getFirstArrayItemOrObject(fields.status);
		card.createdBy = this.getFirstArrayItemOrObject(fields.createdBy);

		var replaceFile = this.getFirstArrayItemOrObject(fields.replaceFile);
		if (replaceFile && 'on'==replaceFile) {
			card.replaceFile = true;
		}

		var replaceOwner = this.getFirstArrayItemOrObject(fields.replaceOwner);
		if (replaceOwner && 'on'==replaceOwner) {
			card.replaceOwner = true;
			card.ownerId = this.getFirstArrayItemOrObject(fields.ownerId);
		}

		if (files && files.file1 && files.file1.length>0) {
			if (clientserverCommon.endsWith(files.file1[0].originalFilename, '.htm') || clientserverCommon.endsWith(files.file1[0].originalFilename, '.html')) {
				this.getAndDeleteFile(files.file1[0].path, function (err, buffer) {
					if (err) {
						return callback(err, card);
					}
					card.file = {
						originalname: files.file1[0].originalFilename,
						path: files.file1[0].path,
						buffer: buffer
					};
					callback(null, card);
				});
			}
			else {
				callback(new Error('wrong file type'), card);
			}
		}
		else {
			callback(null, card);
		}
	},

	getAndDeleteFile: function (path, callback) {
		//Returns err, BinData
		log.debug(path);
		fs.readFile(path, function (err, data) {
			if (err) {
				return callback(err);
			}
			fs.unlink(path, function (err) {
				if (err) {
					return callback(err);
				}
				else {
					callback(null, data);
				}
			});
		});
	},

	getUser: function (req) {
		var user = {};
		user.lastname = req.body.lname;
		user.firstname = req.body.fname;
		user.email = req.body.email;
		user.status = req.body.status;
		user.userType = req.body.userType;
		return user;
	},

	getUser2: function (fields) {
		var user = {};
		user.lastname = this.getFirstArrayItemOrObject(fields.lname);
		user.firstname = this.getFirstArrayItemOrObject(fields.fname);
		user.email = this.getFirstArrayItemOrObject(fields.email);
		user.status = this.getFirstArrayItemOrObject(fields.status);
		user.userType = this.getFirstArrayItemOrObject(fields.userType);
		user.activeDirId = this.getFirstArrayItemOrObject(fields.sysid);
		return user;
	},

	stringArrayToLines: function (ary) {
		if (ary) {
			var s = '';
			for (var i=0;i<ary.length;i++) {
				s += ary[i];
				if (i<ary.length-1) {
					s += '\r\n';
				}
			}
			return s;
		}
		else {
			return '';
		}
	},

	getStatusText: function (text) {
		if (text && 'active'==text) {
			return 'Published';
		}
		else {
			return 'Unpublished';
		};
	},

	getDashDate: function (str) {
		if (!str) return;
		var matches = str.match(/(\d{1,2})-(\d{1,2})-(\d{4})/);
		if (!matches) return;

		// parse each piece and see if it makes a valid date object
		var month = parseInt(matches[1], 10);
		var day = parseInt(matches[2], 10);
		var year = parseInt(matches[3], 10);
		var date = new Date(year, month - 1, day);
		if (!date || !date.getTime()) return;

		// make sure we have no funny rollovers that the date object sometimes accepts
		// month > 12, day > what's allowed for the month
		if (date.getMonth() + 1 != month ||
			date.getFullYear() != year ||
			date.getDate() != day) {
			return;
		}
		return date;
	},

	getTimeDate: function (str) {
		if (!str) return;
		var num = parseInt(str);
		if ('NaN'===num.toString()) {
			return;
		}
		return new Date(num);
	},

	pad: function (n, width, z) {
		z = z || '0';
		n = n + '';
		return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
	},

	formatDateMajorMinor: function (aDate) {
		return aDate.getFullYear() + '-' + clientserverCommon.pad2(aDate.getMonth()+1) + '-' + clientserverCommon.pad2(aDate.getDate()) + ' ' + clientserverCommon.pad2(aDate.getHours()) + ':' + clientserverCommon.pad2(aDate.getMinutes()) + ':' + clientserverCommon.pad2(aDate.getSeconds());
	},

	writeMetricsWithIterator: function (res, iter, callback, isFirst) {
		var helper = this;
		if (isFirst) {
			res.write('"timestamp","deviceHash","ip","request"', 'utf8', function (err) {
				if (err) {
					iter.done();
					callback(err);
				}
				else {
					helper.writeMetricsWithIterator(res,iter,callback);
				}
			});
		}
		else {
			iter.getNext(function (err, item) {
				if (err) {
					iter.done();
					callback(err);
				}
				else if (item) {
					res.write('\r\n"' + helper.formatDateMajorMinor(item.timestamp) + '","' + item.deviceHash + '","' + item.clientIp + '","' + item.request +'"', 'utf8', function (err) {
						if (err) {
							iter.done();
							callback(err);
						}
						else {
							helper.writeMetricsWithIterator(res,iter,callback);
						}
					});
				}
				else {
					iter.done();
					callback();
				}
			});
		}
	},

	writeMetricsWithIterator2: function (res, iter, callback, isFirst) {
		var helper = this;

		if (isFirst) {
			try {
				res.write('"timestamp","deviceHash","ip","request"', 'utf8');
			}
			catch (err) {
				log.debug(err);
				iter.done();
				return callback(err);
			}
			this.writeMetricsWithIterator2(res,iter,callback);
		}
		else {
			iter.getNext(function (err, item) {
				if (err) {
					iter.done();
					callback(err);
				}
				else if (item) {
					try {
						res.write('\r\n"' + helper.formatDateMajorMinor(item.timestamp) + '","' + item.deviceHash + '","' + item.clientIp + '","' + item.request +'"', 'utf8');
					}
					catch (err) {
						iter.done();
						return callback(err);
					}
					helper.writeMetricsWithIterator2(res,iter,callback);
				}
				else {
					iter.done();
					callback();
				}
			});
		}
	},

	getMonthYear: function (aDate) {
		if (!aDate) return '';
		if (!(aDate.getMonth && aDate.getFullYear)) {
			return aDate.toString();
		}
		var result = '';
		switch (aDate.getMonth()) {
			case 0: {
				result+='Jan';
				break;
			}
			case 1: {
				result+='Feb';
				break;
			}
			case 2: {
				result+='Mar';
				break;
			}
			case 3: {
				result+='Apr';
				break;
			}
			case 4: {
				result+='May';
				break;
			}
			case 5: {
				result+='Jun';
				break;
			}
			case 6: {
				result+='Jul';
				break;
			}
			case 7: {
				result+='Aug';
				break;
			}
			case 8: {
				result+='Sep';
				break;
			}
			case 9: {
				result+='Oct';
				break;
			}
			case 10: {
				result+='Nov';
				break;
			}
			case 11: {
				result+='Dec';
				break;
			}
		}
		result += ' ' + aDate.getFullYear();
		return result;
	},

	getSessionState: function (req, silent) {
		if (!req)
			if (silent) {
				return null;
			}
			else {
				throw new Error('request was empty');
			}
		if (!req.session) {
			if (silent) {
				return null;
			}
			else {
				throw new Error('session was empty');
			}
		}
		if (!req.session.stateObject) {
			req.session.stateObject = {};
		}
		return req.session.stateObject;
	},
	
	getExitUrl: function (env, nodeEnv) {
		return this.getIamBasePath(env, nodeEnv, IAM_EXIT_PATH);
	},
	
	getLogoutUrl: function (env, nodeEnv) {
		return this.getIamBasePath(env, nodeEnv, IAM_IDLE_LOGOUT_PATH);
	},
	
	getIamBasePath: function (envObj, nodeEnv, endPath) {
		var base = null;
		if (envObj) {
			var env = envObj.toString().toLowerCase();
			switch (env) {
				case 'dev': {
					base = IAM_DEV_BASE;
					break;
				}
				case 'sqa': {
					base = IAM_SQA_BASE;
					break;
				}
				case 'preprod': {
					base = IAM_PREP_BASE;
					break;
				}
				default: {
				}
			}
		}

		if (!base && nodeEnv && 'production'==nodeEnv) {
			base = IAM_PROD_BASE;
		}

		if (base) {
			return base + IAM_MIDDLE_PATH + endPath;
		}

		return null;
	}

};

module.exports = ui_helper;