/* global Buffer */
/* global afterEach */
/* global beforeEach */
/* global it */
/* global describe */
//Mongo_dao_itest_test.js
//
var should = require('should');
var Mongo_dao = require('../dao/AdminDao_Mongo');
var log4js = require('log4js');
var mclient = require('mongodb').MongoClient;
var log = log4js.getLogger('AdminDao_Mongo_itest_test');
var daoLog = log4js.getLogger('AdminDao_Mongo');
daoLog.setLevel('ERROR');
log.setLevel('ERROR');

//************************************
//	Enter in the test DB info below.
//
//  If the fields in validDbLogin are left blank, their tests will automatically be skipped
//
var validDbLogin = {
	url: '', 
	dbName: '',
	//url: 'mongodb://localhost:27017',
	//dbName: '/pc',
	getConnectString: function () {
		return this.url + this.dbName;
	}
};
//************************************

var invalidDbLogin = {
	url: 'mongodb://localhost:11111', 
	dbName: '/pcpc'
};

describe('Mongo_dao integration tests', function () {
	if (validDbLogin && validDbLogin.url && validDbLogin.dbName) {
		var dao = new Mongo_dao(validDbLogin);

		describe('getCurrentUserBySysid()', function() {
			var adid = 'vhaisx_pcuser1';

			it ('returns a value for a user - no historical duplicates', function (done) {
				dao.getCurrentUserBySysid(adid, function (err, results) {
					if (err) {
						throw err;
					}
					results.length.should.equal(1);
					var result = results[0];
					result.id.should.equal(1);
					result.lastname.should.equal('Joe');
					result.firstname.should.equal('G.I.');
					result.status.should.equal('active');
					result.activeDirId.should.equal(adid);
					result.userType.should.equal('user');
					result.effectiveDate.getTime().should.equal(1396332000000);
					done();
				});
			});

			it ('returns a single user - historical duplicate', function (done) {
				dao.getCurrentUserBySysid('vhaiszxxxx', function (err, results) {
					if (err) {
						throw err;
					}
					results.length.should.equal(1);
					var result = results[0];
					result.id.should.equal(3);
					result.lastname.should.equal('Winchester');
					result.firstname.should.equal('Charles');
					result.status.should.equal('inactive');
					result.activeDirId.should.equal('vhaiszxxxx');
					result.userType.should.equal('card');
					result.effectiveDate.getTime().should.equal(1425830921837);
					done();
				});
			});

			it ('returns nothing if user not found', function (done) {
				dao.getCurrentUserBySysid('fred', function (err, result) {
					if (err) {
						throw err;
					}
					should(result).be.not.ok;
					done();
				});
			});

		});

		describe('saveNewUser()', function () {
			var newUserId = null;

			beforeEach(function (done) {
				_resetNextUserId(function (err, result) {
					if (err) throw err;
					newUserId = result+1;
					done();
				});
			});

			it ('saves a user and return a new ID number', function (done) {
				var nu = {'lastname': 'jone', 'firstname': 'james', 'activeDirId': 'vhaisx_pcuserx', 'status': 'active', 'effectiveDate': new Date('2013-01-01 00:00:00'), 'userType': 'card'};
				dao.saveNewUser({'id': 223344}, nu, function (err, result) {
					if (err) throw err;
					result.should.equal(newUserId);
					_getUsers(newUserId, function (err, results) {
						if (err) throw err;
						results.length.should.equal(1);
						var user = results[0];
						user.lastname.should.equal(nu.lastname);
						user.firstname.should.equal(nu.firstname);
						user.status.should.equal(nu.status);
						user.activeDirId.should.equal(nu.activeDirId);
						should(user.effectiveDate.toString()==nu.effectiveDate.toString()).be.ok;
						user.id.should.equal(newUserId);
						done();
					});
				});
			});

			afterEach(function (done) {
				_removeUser(newUserId, function (err) {
					if (err) throw err;
					_resetNextUserId(function (err) {
						if (err) throw err;
						done();
					});
				});
			});
		});

		describe('getNextId()', function () {

			describe('user ID', function () {
				var newUserId = null;

				beforeEach(function (done) {
					_resetNextUserId(function (err, result) {
						if (err) throw err;
						newUserId = result;
						done();
					});
				});

				it ('returns the next user ID from the "ids" collection', function (done) {
					mclient.connect(validDbLogin.connectString, function (err, db) {
						dao.getNextId(112233, 'user_id', db, function (err, result) {
							should(newUserId+1==result).be.ok;
							db.collection('ids').find({'name': 'user_id', 'value': newUserId+1}).toArray(function (err, results) {
								db.close();
								results.length.should.equal(1);
								should(results[0].locked_by).be.not.ok;
								should(results[0].locked_at).be.not.ok;
								done();
							});
						});
					});
				});

				afterEach(function (done) {
					_resetNextUserId(function (err) {
						if (err) throw err;
						done();
					});
				});
			});

			describe('card ID', function () {
				var newCardId = null;

				beforeEach(function (done) {
					resetNextCardId(function (err, result) {
						if (err) throw err;
						newCardId = result;
						done();
					});
				});

				it ('returns the next card ID from the "ids" collection', function (done) {
					mclient.connect(validDbLogin.connectString, function (err, db) {
						dao.getNextId(112233, 'card_id', db, function (err, result) {
							should(newCardId+1==result).be.ok;
							db.collection('ids').find({'name': 'card_id', 'value': newCardId+1}).toArray(function (err, results) {
								db.close();
								results.length.should.equal(1);
								should(results[0].locked_by).be.not.ok;
								should(results[0].locked_at).be.not.ok;
								done();
							});
						});
					});
				});

				afterEach(function (done) {
					resetNextCardId(function (err) {
						if (err) throw err;
						done();
					});
				});
			});
		});

		describe('recordAction()', function () {
			it ('saves the record', function (done) {
				var aDate = new Date();
				var actionRec = {'userId': 112233, 'cardId': 332211, 'action': 'something', 'datetime': aDate};
				dao.recordAction(actionRec, function (err) {
					if (err) {
						throw err;
					}
					_getAudits({'userId': 112233, 'cardId': 332211}, function (err, result) {
						if (err) throw err;
						result.length.should.equal(1);
						done();
					});
				});
			});

			afterEach(function (done) {
				_clearAudits({'userId': 112233, 'cardId': 332211, 'action': 'something'}, function (err) {
					if (err) throw err;
					done();
				});
			});
		});

		describe('getCurrentUserById()', function () {
			it ('finds the correct record', function (done) {
				dao.getCurrentUserById(3, function (err, result) {
					if (err) throw err;
					log.debug('getCurrentUserById() result:');
					log.debug(result);
					result.lastname.should.equal('Winchester');
					result.status.should.equal('inactive');
					result.effectiveDate.getTime().should.equal(1425830921837);
					done();
				});
			});
		});

		describe('saveNewCard()', function () {
			var expectedCardId = 0;

			beforeEach(function (done) {
				resetNextCardId(function (err, highestCardId) {
					expectedCardId = highestCardId + 1;
					done();
				});
			});

			it ('saves new card', function (done) {
				var user = {id:112233};
				var data = {
					ownerId: 2,
					title: 'blah',
					descr: 'blah blah',
					tags: ['blah'],
					file: '<html></html>',
					version: '1.5.374892',
					versionComment: 'hello',
					status: 'active',
					effectiveDate: new Date()
				};
				dao.saveNewCard(user, data, function (err, newCardId) {
					if (err) {
						throw err;
					}
					log.debug("saveNewCard new ID=");
					log.debug(newCardId);
					expectedCardId.should.equal(newCardId);
					done();
				});
			});

			afterEach(function (done) {
				mclient.connect(validDbLogin.connectString, function (err, db) {
					if (err) throw err;
					db.collection('cards').remove({id: expectedCardId}, function (err, results) {
						if (err) throw err;
						resetNextCardId(function() {
							done();
						});
					});
				});
			});
		});

		describe('getCurrentCard()', function () {
			it ('returns current card data', function (done) {
				dao.getCurrentCard(1, function (err, result) {
					result.id.should.equal(1);
					result.version.should.equal('1.0.1');
					should(result.effectiveDate.getTimezoneOffset).be.ok;
					result.effectiveDate.getTime().should.equal(1416294000000);
					done();
				});
			});
		});

		describe('getCurrentCards()', function () {
			it ('returns all without ownerId', function (done) {
				dao.getCurrentCards(null, function (err, result) {
					validate3Cards(result);
					done();
				});
			});

			it ('returns owned cards1', function (done) {
				dao.getCurrentCards(1, function (err, result) {
					validate3Cards(result);
					done();
				});
			});

			it ('returns owned cards2', function (done) {
				dao.getCurrentCards(2, function (err, results) {
					results.length.should.equal(0);
					done();
				});
			});
		});

		describe('updateCard()', function () {
			it ('happy path', function (done) {
				var card = getTestCard();
				dao.updateCard(card, function (err) {
					if (err) throw err;
					mclient.connect(validDbLogin.connectString, function (err, db) {
						db.collection('cards').find({id:10101}).toArray(function (err, result) {
							db.close();
							if (err) throw err;
							if (!Array.isArray(result)) {
								throw new Error('wrong result');
							}
							result.length.should.equal(1);
							var tc = result[0];
							tc.id.should.equal(card.id);
							tc.ownerId.should.equal(card.ownerId);
							tc.title.should.equal(card.title);
							tc.descr.should.equal(card.descr);
							tc.version.should.equal(card.version);
							tc.versionComment.should.equal(card.versionComment);
							tc.status.should.equal(card.status);
							tc.effectiveDate.toString().should.equal(card.effectiveDate.toString());
							JSON.stringify(tc.file).should.equal(JSON.stringify(card.file));
							done();
						});
					});
				});
			});

			afterEach(function (done) {
				mclient.connect(validDbLogin.connectString, function (err, db) {
					db.collection('cards').remove({id:10101}, function (err, result) {
						db.close();
						if (err) throw err;
						log.debug('after updateCard() test result:');
						log.debug(result);
						if (1!=result) {
							throw new Error('test card not removed from database!');
						}
						done();
					});
				});
			});
		});

		describe('getUsers()', function () {
			it ('happy path', function (done) {
				dao.getUsers(function (err, results) {
					results.length.should.equal(6);
					assertHasUser(results, 1, 'Joe', 'G.I.', 'vhaisx_pcuser1', 'active', 'user', new Date('2014-04-01 00:00:00'))
					assertHasUser(results, 2, 'Everhart', 'Braxton', 'vhaisx_pcuser2', 'active', 'card', new Date('2014-01-01 00:00:00'));
					assertHasUser(results, 3, 'Winchester', 'Charles', 'vhaiszxxxx', 'inactive', 'card', new Date('2015-03-08T16:08:41.837Z'));
					assertHasUser(results, 4, 'Flintstone', 'Fred', 'vhaiszxxxx2', 'inactive', 'card', new Date('2015-02-08T20:00:00.000Z'))
					assertHasUser(results, 5, 'Onelast', 'Onefirst', 'vhaisltest', 'inactive', 'card', new Date('2015-05-08T16:08:41.837Z'))
					assertHasUser(results, 6, 'Twolast', 'Twofirst', 'vhaisltest', 'inactive', 'user', new Date('2015-05-08T20:00:00.000Z'))
					done();
				});
			});
		});

		describe('updateUser()', function () {
			var newUser = {
				id:3,
				lastname:'Winchester',
				firstname:'Charles',
				status:'active',
				email:'ce.winchester@abc.defg',
				activeDirId:'vhaiszxxxx1',
				effectiveDate: new Date('2015-02-01 07:08:09')
			};

			it ('happy path', function (done) {
				dao.updateUser(newUser, function (err, result) {
					if (err) throw err;
					mclient.connect(validDbLogin.connectString, function (err, db) {
						if (err) {
							db.close();
							throw err;
						}
						db.collection('adminUsers').find({id: newUser.id, effectiveDate: newUser.effectiveDate}).toArray(function (err, ary) {
							db.close();
							if (err) throw err;
							ary.length.should.equal(1);
							var user = ary[0];
							assertSame(newUser, user, ['id', 'lastname', 'firstname', 'status', 'email', 'activeDirId']);
							newUser.effectiveDate.getTime().should.equal(user.effectiveDate.getTime());
							done();
						});
					});
				});
			});

			afterEach(function (done) {
				mclient.connect(validDbLogin.connectString, function (err, db) {
					if (err) {
						db.close();
						throw err;
					}
					db.collection('adminUsers').remove({id:newUser.id, effectiveDate:newUser.effectiveDate}, function (err, result) {
						db.close();
						if (err) throw err;
						done();
					});
				});
			});
		});

		describe('getCardFile()', function () {
			it ('happy path', function (done) {
				dao.getCardFile(2, function (err, result) {
					if (err) {
						throw err;
					}
					result.originalname.should.equal('small.html');
					should(Buffer.isBuffer(result.buffer.buffer)).be.ok;
					done();
				});
			});

			it ('handles empty result', function (done) {
				dao.getCardFile('fred', function (err, result) {
					if (err) {
						throw err;
					}
					should(result).be.not.ok;
					done();
				});
			});
		});

		describe('getCardHistory()', function () {
			it ('handles empty result', function (done) {
				dao.getCardHistory(10101, function (err, results) {
					if (err) throw  err;
					if (results) {
						results.length.should.equal(0);
					}
					done();
				});
			});

			it ('happy path', function (done) {
				dao.getCardHistory(2, function (err, cards) {
					cards.length.should.equal(3);
					cards.sort(effectiveDateArraySort);
					cards[0].id.should.equal(2);
					cards[0].versionComment.should.equal('Initial release');
					var dt = cards[0].effectiveDate;
					dt.getDate().should.equal(1);
					dt.getFullYear().should.equal(2014);
					dt.getMonth().should.equal(0);

					cards[1].id.should.equal(2);
					cards[1].versionComment.should.equal('Added formatting');
					dt = cards[1].effectiveDate;
					dt.getDate().should.equal(1);
					dt.getFullYear().should.equal(2014);
					dt.getMonth().should.equal(2);

					cards[2].id.should.equal(2);
					cards[2].versionComment.should.equal('Changed title, added content');
					dt = cards[2].effectiveDate;
					dt.getDate().should.equal(1);
					dt.getFullYear().should.equal(2014);
					dt.getMonth().should.equal(4);
					done();
				});
			});
		});

		describe('getCardAuditHistory()', function () {
			it ('handles empty result', function (done) {
				dao.getCardAuditHistory(10101, function (err, results) {
					if (err) throw err;
					if (results) {
						results.length.should.equal(0);
					}
					done();
				});
			});

			it ('happy path', function (done) {
				dao.getCardAuditHistory(2, function (err, hist) {
					if (err) throw err;
					hist.length.should.equal(3);
					hist.sort(effectiveDateArraySort);
					assertAudit(hist[0], 2, 1, 1, 1, 2014);
					assertAudit(hist[1], 2, 2, 3, 1, 2014);
					assertAudit(hist[2], 2, 2, 5, 1, 2014);
					done();
				});
			});
		});

		describe('setPersonNames()', function () {
			it ('handles null input object', function (done) {
				dao.setPersonNames(null, function (err) {
					if (err) throw err;
					done();
				});
			});

			it ('handles empty input object', function (done) {
				var ob = {}
				dao.setPersonNames(ob, function (err) {
					if (err) throw err;
					var keys = Object.getOwnPropertyNames(ob);
					if (keys) {
						keys.length.should.equal(0);
					}
					done();
				});
			});

			it ('happy path', function (done) {
				var ob = {1:1, '3':1};
				dao.setPersonNames(ob, function(err) {
					if (err) throw err;
					var keys = Object.getOwnPropertyNames(ob);
					keys.length.should.equal(2);
					ob[1].should.equal('G.I. Joe');
					ob[3].should.equal('Charles Winchester');
					done();
				});
			});
		});

		describe('getCardFileInstance()', function () {
			it ('handles not found card files', function (done) {
				dao.getCardFileInstance(10101, 'abcd1234abcd1234abcd1234', function (err, result) {
					if (err) throw err;
					should(result).not.be.ok;
					done();
				});
			});

			it ('throws error if instanceId blank', function (done) {
				dao.getCardFileInstance(10101, null, function (err, result) {
					err.toString().should.equal('Error: mdao: no instanceId given');
					should(result).be.not.ok;
					done();
				});
			});

			it ('throws error if instanceId not 24 characters', function (done) {
				dao.getCardFileInstance(10101, '112233', function (err, result) {
					err.toString().should.equal('Error: mdao: instanceId wrong length');
					should(result).be.not.ok;
					done();
				});
			});

			it ('happy path', function (done) {
				dao.getCardFileInstance(2, '5500b6660a126a09cbf10184', function (err, result) {
					if (err) throw err;
					result.originalname.should.equal('small.html');
					should(Buffer.isBuffer(result.buffer.buffer)).be.ok;
					done();
				});
			});
		});

		describe('getAllCurrentUserNames()', function () {
			it ('happy path', function (done) {
				dao.getAllCurrentUserNames(function (err, users) {
					if (err) throw err;
					users.length.should.equal(6);
					for (var i = users.length - 1; i >= 0; i--) {
						Object.getOwnPropertyNames(users[i]).length.should.equal(3);
					};
					users.sort(function (i1, i2) {
						return i1.id-i2.id;
					});
					users[0].id.should.equal(1);
					users[0].lastname.should.equal('Joe');
					users[0].firstname.should.equal('G.I.');

					users[1].id.should.equal(2);
					users[1].lastname.should.equal('Everhart');
					users[1].firstname.should.equal('Braxton');

					users[2].id.should.equal(3);
					users[2].lastname.should.equal('Winchester');
					users[2].firstname.should.equal('Charles');

					users[3].id.should.equal(4);
					users[3].lastname.should.equal('Flintstone');
					users[3].firstname.should.equal('Fred');

					users[4].id.should.equal(5);
					users[4].lastname.should.equal('Onelast');
					users[4].firstname.should.equal('Onefirst');

					users[5].id.should.equal(6);
					users[5].lastname.should.equal('Twolast');
					users[5].firstname.should.equal('Twofirst');

					done();
				});
			});
		});

		describe('getMetaMetricsIterator()', function () {
			describe ('iterator with no items', function () {

				it ('returns iterator that returns nothing', function (done) {
					dao.getMetaMetricsIterator(new Date('1/1/2010'), new Date('12/31/2010'), function (err, iter) {
						if (err) throw err;
						iter.getNext(function (item) {
							should(item).be.not.ok;
							iter.done();
							iter.getNext(function (item2) {
								should(item2).be.not.ok;
								done();
							});
						});
					});
				});

			});

			describe('iterator with some items', function () {
				beforeEach(function (done) {
					setupTestMetrics(done);
				});

				it ('reads expected metrics', function (done) {
					dao.getMetaMetricsIterator(new Date('3/31/2012'), new Date('4/4/2012'), function (err, iter) {
						if (err) throw err;
						var items = [];
						iter.getNext(function (err, item) {
							if (err) throw err;
							if (item) {
								items.push(item);
							}
							iter.getNext(function (err2, item2) {
								if (err2) throw err2;
								if (item2) {
									items.push(item2);
								}
								iter.getNext(function (err3, item3) {
									if (err3) throw err3;
									if (item3) {
										items.push(item3);
									}
									iter.done();
									items.length.should.equal(2);
									items.sort(timestampSort);
									items[0].deviceHash.should.equal('abcdefghijk');
									items[0].clientIp.should.equal('127.0.0.1');
									items[0].request.should.equal('/pocketcardv1/user/abcdefghijk/cards');
									items[1].deviceHash.should.equal('abcdefghijk');
									items[1].clientIp.should.equal('127.0.0.1');
									items[1].request.should.equal('/pocketcardv1/user/abcdefghijk/cards?state=active');
									done();
								});
							})
						});
					});
				});

				it ('returns nothing after \'done\' is called', function (done) {
					dao.getMetaMetricsIterator(new Date('3/31/2012'), new Date('4/4/2012'), function (err, iter) {
						if (err) throw err;
						var items = [];
						iter.getNext(function (err, item) {
							if (err) throw err;
							if (item) {
								items.push(item);
							}
							iter.done();
							iter.getNext(function (err2, item2) {
								if (err2) throw err2;
								if (item2) {
									items.push(item2);
								}
								iter.getNext(function (err3, item3) {
									if (err3) throw err3;
									if (item3) {
										items.push(item3);
									}
									items.length.should.equal(1);
									done();
								});
							})
						});
					});
				});

				afterEach(function (done) {
					removeTestMetrics(done);
				});
			});
		});

		describe('getCardMetricsIterator()', function () {

			describe('iterator with no items', function () {
				it ('returns nothing', function (done) {
					dao.getCardMetricsIterator('238472389472', null, null, function (err, iter) {
						if (err) throw err;
						iter.getNext(function (err, item) {
							if (err) throw err;
							should(item).be.not.ok;
							iter.done();
							done();
						});
					});
				});
			});

			describe('iterator with some items', function () {
				beforeEach(function (done) {
					setupTestMetrics(done);
				});

				it ('returns all items from a card (no dates)', function (done) {
					dao.getCardMetricsIterator('10102', null, null, function (err, iter) {
						if (err) throw err;
						getNextUntilEmptySink(iter, function (items) {
							items.length.should.equal(3);
							items.sort(timestampSort);
							items.forEach(function (item) {
								item.request.should.equal('/pocketcardv1/user/abcxyz112233/cards/10102/contents');
							});
							items[0].timestamp.getMonth().should.equal(3);
							items[1].timestamp.getMonth().should.equal(3);
							items[2].timestamp.getMonth().should.equal(5);
							iter.done();
							done();
						});
					});
				});

				it ('returns all items from a card (with dates)', function (done) {
					dao.getCardMetricsIterator('10101', new Date('4-1-2012 10:00:00'), new Date('4-2-2012'), function (err, iter) {
						if (err) throw err;
						getNextUntilEmptySink(iter, function (items) {
							items.length.should.equal(2);
							items.sort(timestampSort);
							items.forEach(function (item) {
								item.request.should.equal('/pocketcardv1/user/abcxyz112233/cards/10101/contents');
							});
							items[0].timestamp.getMonth().should.equal(3);
							items[1].timestamp.getMonth().should.equal(3);
							iter.done();
							done();
						});
					});
				});

				it ('returns all items from all cards (no dates)', function (done) {
					dao.getCardMetricsIterator(null, null, null, function (err, iter) {
						if (err) throw err;
						getNextUntilEmptySink(iter, function (items) {
							items.length.should.equal(6);
							items.sort(timestampSort);
							iter.done();
							var tItems = getSampleMetrics().slice(3,100);
							for (var i = items.length - 1; i >= 0; i--) {
								items[i].request.should.equal(tItems[i].request);
								items[i].timestamp.getTime().should.equal(tItems[i].timestamp.getTime());
							};
							done();
						});
					});
				});

				it ('returns nothing after \'done\' is called', function (done) {
					dao.getCardMetricsIterator('10102', null, null, function (err, iter) {
						if (err) throw err;
						iter.done();
						iter.getNext(function (err, item) {
							if (err) throw err;
							should(item).be.not.ok;
							done();
						});
					});
				});

				afterEach(function (done) {
					removeTestMetrics(done);
				});
			});

		});

		describe('getCurrentUserIdsBySysid()', function () {
			var adid = 'vhaisx_pcuser1';

			it ('returns a value for a user - no historical duplicates', function (done) {
				dao.getCurrentUserIdsBySysid(adid, function (err, results) {
					if (err) {
						throw err;
					}
					results.length.should.equal(1);
					results[0].should.equal(1);
					done();
				});
			});

			it ('returns a value for a user - valid historical duplicates exist', function (done) {
				dao.getCurrentUserIdsBySysid('vhaiszxxxx', function (err, results) {
					if (err) {
						throw err;
					}
					results.length.should.equal(1);
					results[0].should.equal(3);
					done();
				});
			});

			it ('returns duplicates', function (done) {
				dao.getCurrentUserIdsBySysid('vhaisltest', function (err, results) {
					if (err) {
						throw err;
					}
					results.length.should.equal(2);
					results.sort();
					results[0].should.equal(5);
					results[1].should.equal(6);
					done();
				});
			});

			it ('returns nothing if user not found', function (done) {
				dao.getCurrentUserIdsBySysid('fred', function (err, result) {
					if (err) {
						throw err;
					}
					should(result).be.not.ok;
					done();
				});
			});
		});

	}
	else {
		log.fatal('***********************************************************');
		log.fatal('*  Some AdminDao_Mongo_itest_test  tests not run          *');
		log.fatal('*                                                         *');
		log.fatal('* In order to run these tests, you must configure this    *');
		log.fatal('* file to point to a Mongo instance with the test data    *');
		log.fatal('* found in                                                *');
		log.fatal('* `data/mongo/pocketCard_insert_SampleCards.mongo`        *');
		log.fatal('*                                                         *');
		log.fatal('***********************************************************');
	}

	describe('Invalid connection handling tests', function () {
		var fakePlugin = {
			connect: function (parm1, parm2, parm3) {
				var callback;
				if (parm3) {
					callback = parm3;
				}
				else {
					callback = parm2;
				}
				callback(new Error('failed to connect'));
			}
		};
		var dao = new Mongo_dao(invalidDbLogin, fakePlugin);
		var msets = [
			{
				names: [
					'getUsers',
					'getAllCurrentUserNames'
				],
				pcount: 1
			},
			{
				names: [
					'getCurrentUserBySysid',
					'getCurrentUserIdsBySysid',
					'recordAction',
					'updateUser',
					'getCurrentUserById',
					'updateCard',
					'getCurrentCard',
					'getCurrentCards',
					'getCardFile',
					'getCardHistory',
					'getCardAuditHistory',
					'setPersonNames'
				],
				params: {setPersonNames: [{1:'stuff'}]},
				pcount: 2
			},
			{
				names: [
					'saveNewUser',
					'saveNewCard',
					'getCardFileInstance',
					'getMetaMetricsIterator'
				],
				params: {getMetaMetricsIterator: [new Date(), new Date()], getCardFileInstance: ['stuff', '123456789012345678901234']},
				pcount:3
			},
			{
				names: [
					'getCardMetricsIterator'
				],
				pcount:4
			}
		];

		for (var i = 0; i<msets.length; i++) {
			var parmCount = msets[i].pcount;
			for (var j=0;j<msets[i].names.length;j++) {
				var method = msets[i].names[j];
				var parms = null;
				if (msets[i].params && msets[i].params[method]) {
					parms = msets[i].params[method];
				}
				else {
					parms = ['stuff', 'stuff', 'stuff'];
				}
				describe (method, function () {
					var name = method;
					var params = parms;
					switch (parmCount) {
						case 1: {
							it ('handles connection error (1 param)', function (done) {
								dao[name](function (err, value) {
									should(err).be.ok;
									var index = err.message.indexOf('failed to connect');
									if (index<0) {
										log.debug(err);
										throw new Error('wrong message:' + err.message);
									}
									should(value).be.not.ok;
									done();
								});
							});
							break;
						}
						case 2: {
							it ('handles connection error (2 params)', function (done) {
								dao[name](params[0], function (err, value) {
									should(err).be.ok;
									var index = err.message.indexOf('failed to connect');
									if (index<0) {
										log.debug(err);
										throw new Error('wrong message:' + err.message);
									}
									should(value).be.not.ok;
									done();
								});
							});
							break;
						}
						case 3: {
							it ('handles connection error (3 params)', function (done) {
								dao[name](params[0], params[1], function (err, value) {
									should(err).be.ok;
									var index = err.message.indexOf('failed to connect');
									if (index<0) {
										log.debug(err);
										throw new Error('wrong message:' + err.message);
									}
									should(value).be.not.ok;
									done();
								});
							});
							break;
						}
						case 4: {
							it ('handles connection error (4 params)', function (done) {
								dao[name](params[0], params[1], params[2], function (err, value) {
									should(err).be.ok;
									var index = err.message.indexOf('failed to connect');
									if (index<0) {
										log.debug(err);
										throw new Error('wrong message:' + err.message);
									}
									should(value).be.not.ok;
									done();
								});
							});
							break;
						}
						default: {}
					}
				});
			}
		};

	});
	//TODO: add bad connection tests :)
});

function assertAudit(audit, cardId, userId, month, day, year) {
	audit.itemCardId.should.equal(cardId);
	audit.userId.should.equal(userId);
	audit.effectiveDate.getMonth().should.equal(month-1);
	audit.effectiveDate.getDate().should.equal(day);
	audit.effectiveDate.getFullYear().should.equal(year);
}

function effectiveDateArraySort(o1, o2) {
	return o1.effectiveDate.getTime()-o2.effectiveDate.getTime();
}

function timestampSort(o1, o2) {
	return o1.timestamp.getTime()-o2.timestamp.getTime();
}

function assertSame(obj1, obj2, propArray) {
	if (obj1 && obj2) {
		for (var i = propArray.length - 1; i >= 0; i--) {
			if (obj1[propArray[i]]) {
				obj1[propArray[i]].should.equal(obj2[propArray[i]]);
			}
			else {
				should(obj2[propArray[i]]).be.not.ok;
			}
		};
		return;
	}
	if (obj1 && !obj2) {
		throw new Error('2nd object null');
	}
	throw new Error('1st object null');
}

function assertHasUser(ary, id, lname, fname, adid, status, utype, effDt) {
	log.debug('assertHasUser ary==');
	log.debug(ary);
	for (var i = ary.length - 1; i >= 0; i--) {
		if (id==ary[i].id) {
			var user = ary[i];
			user.lastname.should.equal(lname);
			user.firstname.should.equal(fname);
			user.activeDirId.should.equal(adid);
			user.status.should.equal(status);
			user.userType.should.equal(utype);
			user.effectiveDate.getTime().should.equal(effDt.getTime());
			return;
		}
	};
	throw new Error('ID ' + id + ' not found');
}

function getTestCard(effDate) {
	if (!effDate) {
		effDate = new Date();
	}
	return {
		id: 10101,
		ownerId: 90909,
		title: 'title1',
		descr: 'descr1',
		version: '0.0.0.1',
		versionComment: 'vComment1',
		status: 'active',
		effectiveDate: effDate,
		file: {
			fieldname: "file1",
			originalname: "small.html",
			name: "small1419377854423.html",
			encoding: "7bit",
			mimetype: "text/html",
			path: "uploads/small1419377854423.html",
			extension: "html",
			size: 88,
			truncated: false,
			buffer: [32, 32, 32]
		}
	};
}

function validate3Cards(cardAry) {
	cardAry.length.should.equal(3);
	var c1 = cardAry[0];
	var c2 = cardAry[1];
	var c3 = cardAry[2];
	var c0;
	if (c2.id==1) {
		c0 = c2;
		c2 = c1;
		c1 = c2;
	}
	else if (c3.id==1) {
		c0 = c3;
		c3 = c1;
		c1 = c3;
	}
	if (c2.id==3) {
		c0 = c2;
		c2 = c3;
		c3 = c0;
	}
	c1.id.should.equal(1);
	c2.id.should.equal(2);
	c3.id.should.equal(3);
	c1.status.should.equal('inactive');
	c2.status.should.equal('active');
	c3.status.should.equal('inactive');
}

function _removeUser (id, callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) {
			db.close();
			return callback(err);
		}
		db.collection('adminUsers').remove({'id': id}, function (err, result) {
			db.close();
			if (err) {
				log.error(err);
			}
			log.debug('_removeUser result=');
			log.debug(result);
			callback(err, result);
		});
	});
};

function _getUsers(id, callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) {
			db.close();
			return callback(err);
		}
		db.collection('adminUsers').find({'id': id}).toArray(function (err, results) {
			db.close();
			callback(err, results);
		});
	});
};

function _getAudits(search, callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) {
			db.close();
			return callback(err);
		}
		db.collection('audits').find(search).toArray(function (err, results) {
			db.close();
			callback(err, results);
		});
	});
};

function _clearAudits(search, callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) {
			db.close();
			return callback(err);
		}
		db.collection('audits').remove(search, function (err, result) {
			db.close();
			if (err) return callback(err);
			log.debug('_clearAudits result=');
			log.debug(result);
			callback();
		});
	});
};

function _resetNextUserId(callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) {
			db.close();
			return callback(err);
		}
		getMaxUserId(db, function (err, userId) {
			if (err) {
				db.close();
				return callback(err);
			}
			replaceNameId(db, userId, function (err) {
				db.close();
				callback(err, userId);
			});
		});
	});
};

function resetNextCardId(callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) {
			db.close();
			return callback(err);
		}
		getMaxCardId(db, function (err, cardId) {
			if (err) {
				db.close();
				return callback(err);
			}
			replaceCardId(db, cardId, function (err) {
				db.close();
				callback(err, cardId);
			});
		});
	});
}

function getMaxUserId(db, callback) {
	getMaxId(db, 'adminUsers', callback);
}

function getMaxCardId(db, callback) {
	getMaxId(db, 'cards', callback);
}

function getMaxId(db, coll, callback) {
	db.collection(coll).find().sort([['id', -1]]).limit(1).toArray(function (err, result) {
		if (err) return callback(err);
		//log.debug("getMaxId for " + coll + " result=");
		//log.debug(result);
		callback(null, result[0].id);
	});
}

function replaceNameId(db, id, callback) {
	replaceId(db, id, 'user_id', callback);
}

function replaceCardId(db, id, callback) {
	replaceId(db, id, 'card_id', callback);
}

function replaceId(db, id, idType, callback) {
	db.collection('ids').update({'name': idType}, {'name': idType, 'value': id}, function (err, result) {
		if (err) return callback(err);
		callback();
	});
}

function setupTestMetrics(callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) throw err;
		var data = getSampleMetrics();
		db.collection('metrics').insert(data, function (err, result) {
			db.close();
			if (err) throw err;
			callback();
		});
	});
}

function getSampleMetrics() {
	return [
		{deviceHash : "abcdefghijk", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcdefghijk/cards", timestamp : new Date("2012-04-01 15:58:44")},
		{deviceHash : "abcdefghijk", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcdefghijk/cards?state=active", timestamp : new Date("2012-04-01 15:58:45")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10101", timestamp : new Date("2012-04-01 15:58:46")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10101/contents", timestamp : new Date("2012-04-01 15:58:47")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10102/contents", timestamp : new Date("2012-04-01 15:58:48")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10101/contents", timestamp : new Date("2012-04-01 20:58:47")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10102/contents", timestamp : new Date("2012-04-01 20:58:48")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10101/contents", timestamp : new Date("2012-05-01 15:58:47")},
		{deviceHash : "abcxyz112233", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10102/contents", timestamp : new Date("2012-06-01 15:58:48")}
	];
}

function removeTestMetrics(callback) {
	mclient.connect(validDbLogin.connectString, function (err, db) {
		if (err) throw err;
		db.collection('metrics').remove({timestamp: {$gte:new Date('2012-01-01'), $lte:new Date('2012-06-02')}}, function (err, result) {
			db.close();
			if (err) throw err;
			var num = getSampleMetrics().length;
			if (num!=result) {
				throw new Error('remove count unexpected: wanted ' + num + ' got ' + result);
			}
			callback();
		});
	});
}

function getNextUntilEmptySink(iter, callback, items) {
	if (!items) {
		items = [];
	}
	iter.getNext(function (err, item) {
		if (item) {
			items.push(item);
			getNextUntilEmptySink(iter, callback, items);
		}
		else {
			callback(items);
		}
	});
}
