/* global clientserverCommon */
"use strict";
describe("admin web client tests", function () {

	beforeEach(module('adminapp'));
	beforeEach(module(function ($provide) {
		$provide.value('$window', {});
	}));


	describe('adminapp', function () {
		var utils,$filter,$controller;

		beforeEach(inject(function (_$controller_, _$filter_) {
			$controller = _$controller_;
			$filter = _$filter_;
		}));

		describe('utils', function () {
			beforeEach(inject(function (_utils_) {
				utils = _utils_;
			}));

			it ('calculates lear years correctly', function () {
				var leaps = [2000, 2004, 2008, 2012, 2016, 2020, 2024, 2028, 2032];
				for (var i=2000; i<2036; i++) {
					if (_.contains(leaps, i)) {
						expect(utils.isLeapYear(i)).toEqual(true);
					}
					else {
						expect(utils.isLeapYear(i)).toEqual(false);
					}
				}
			});
		});

		describe('filters', function () {
			it ('statustext', function () {
				var statustext = $filter('statustext');
				expect(statustext).not.toBeNull();
				expect(statustext('active')).toEqual('Published');
				expect(statustext('Active')).toEqual('Unpublished');
				expect(statustext('activE')).toEqual('Unpublished');
			});

			it ('timeToString', function () {
				var timeToString = $filter('timeToString');
				expect(timeToString).not.toBeNull();
				expect(timeToString(new Date('1/1/2015 00:00:00'))).toEqual('1420095600000');
				expect(timeToString('fred')).toEqual('fred');
			});

			it ('datetimeString', function () {
				var datetimeString = $filter('datetimeString');
				var dt = new Date('2015-01-08T16:08:41.837Z');
				expect(datetimeString).not.toBeNull();
				var offset = new Date().getTimezoneOffset()/60;
				expect(datetimeString(dt)).toEqual('2015-01-08 ' + clientserverCommon.pad2(15-offset) + ':08');
			});

			it ('datefulltimeString', function () {
				var dfts = $filter('datefulltimeString');
				expect(dfts).not.toBeNull();
				var dt = new Date('2015-01-08T16:08:41.837Z');
				var offset = new Date().getTimezoneOffset()/60;
				expect(dfts(dt)).toEqual('2015-01-08 ' + clientserverCommon.pad2(15-offset) + ':08:41');
			});
		});

		describe('users controller', function() {
			var $httpBackend, $rootScope, userGetGood, createController;

			function getTestUsers () {
				return [
					{"_id" : "5581b67422bfca2b84cd51e4", "id" : 1, "lastname" : "Joe Jr.", "firstname" : "G.I.", "activeDirId" : "vhaisx_pcuser1", "status" : "active", "userType" : "user", "effectiveDate" : "2014-01-01T07:00:00Z"},
					{ "_id" : "5581b67422bfca2b84cd51e6", "id" : 2, "lastname" : "Everhart", "firstname" : "Braxton", "activeDirId" : "vhaisx_pcuser2", "status" : "active", "userType" : "card", "effectiveDate" : "2014-01-01T07:00:00Z"},
					{ "_id" : "5581b67422bfca2b84cd51e7", "id" : 3, "firstname" : "Charles", "lastname" : "Winchester", "activeDirId" : "vhaiszxxxx", "userType" : "card", "status" : "inactive", "effectiveDate" : "2015-01-08T16:08:41.837Z"}
				];
			}

			beforeEach(inject(function ($injector) {
				$httpBackend = $injector.get('$httpBackend');
				userGetGood = $httpBackend.when('GET', 'users').respond(getTestUsers());
				$rootScope = $injector.get('$rootScope');
				createController = function () {
					return $controller('users', {'$scope': $rootScope});
				};
			}));

			it ('initializes as expected', function () {
				$httpBackend.expectGET('users');
				createController();
				$httpBackend.flush();
				expect($rootScope.loading).toEqual(false);
				expect($rootScope.error).toEqual(false);
				expect($rootScope.hasData).toEqual(true);
				expect($rootScope.isEmpty).toEqual(false);
				expect($rootScope.users.length).toEqual(3);
				var dt = new Date();
				expect($rootScope.users[2].effectiveDate.getHours()).toEqual(15-(dt.getTimezoneOffset()/60));
				expect($rootScope.users[2].effectiveDate.getMinutes()).toEqual(8);
			});

			afterEach(function() {
				$httpBackend.verifyNoOutstandingExpectation();
				$httpBackend.verifyNoOutstandingRequest();
			});
		});

		describe('addCardController', function () {
			var $rootScope, acc;

			beforeEach(inject(function ($injector) {
				$rootScope = $injector.get('$rootScope');
				acc = $controller('addCardController', {'$scope': $rootScope});
			}));

			it ('initializes as expected', function () {
				expect($rootScope.status).toEqual('inactive');
				expect($rootScope.lastFileTryError).toEqual(false);
			});

			describe('setFile()', function () {
				it ('sets file name with .htm filename and status unchanged', function () {
					$rootScope.setFile('something.htm');
					expect($rootScope.file).toEqual('something.htm');
					expect($rootScope.status).toEqual('inactive');
					expect($rootScope.lastFileTryError).toEqual(false);
				});

				it ('sets file name with .html filename and status unchanged', function () {
					$rootScope.setFile('something.html');
					expect($rootScope.file).toEqual('something.html');
					expect($rootScope.status).toEqual('inactive');
					expect($rootScope.lastFileTryError).toEqual(false);
				});

				it ('sets file name null with wrong extension and status unchanged as inactive', function () {
					$rootScope.setFile('fred.fred');
					clearFilenameTests($rootScope);
					expect($rootScope.lastFileTryError).toEqual(true);
				});

				it ('sets file name null with wrong extension and status changed to inactive', function () {
					$rootScope.status = 'active';
					$rootScope.setFile('fred.fred');
					clearFilenameTests($rootScope);
					expect($rootScope.lastFileTryError).toEqual(true);
				});

				it ('sets blank file name and status to inactive for empty file name', function () {
					$rootScope.setFile();
					clearFilenameTests($rootScope);
				});

				it ('sets blank file name and status to inactive for empty file name', function () {
					$rootScope.file = 'fred.htm';
					$rootScope.status = 'active';
					$rootScope.setFile();
					clearFilenameTests($rootScope);
				});

				function clearFilenameTests(scope) {
					expect(scope.file).toBeNull()
					expect(scope.status).toEqual('inactive');
				}
			});

		});

		describe('editcardController', function () {
			var $httpBackend, carddataRequestHandler, cardversionsRequestHandler, createController, $rootScope;

			beforeEach(inject(function ($injector) {
				$httpBackend = $injector.get('$httpBackend');
				carddataRequestHandler = $httpBackend.when('GET', 'carddata')
					.respond({title:'a card', descr: 'a descr', tags: ['hi', 'there'], status: 'active', version: '1.2.3', versionComment: 'howdy', createdBy: 'a guy', file: {originalname: 'afile.html'}});
				cardversionsRequestHandler = $httpBackend.when('GET', 'cardversions')
					.respond(['1','2','3']);
				$rootScope = $injector.get('$rootScope');
				createController = function () {
					return $controller('editcardController', {'$scope': $rootScope});
				};
			}));

			it ('initializes as expected', function () {
				$httpBackend.expectGET('cardversions');
				$httpBackend.expectGET('carddata');
				createController();
				$httpBackend.flush();
				expect($rootScope.replaceFile).toEqual(false);
				expect($rootScope.loading).toEqual(false);
				expect($rootScope.card.tags).toEqual('hi\r\nthere');
				expect($rootScope.card.status).toEqual('active');
				expect($rootScope.oldStatus).toEqual('active');
				expect($rootScope.error).toBe(false);
				expect($rootScope.versionLoadError).not.toBeDefined();
			});

			it ('evaluates isVersionStateOkay correctly', function () {
				createController();
				$httpBackend.flush();
				expect($rootScope.card.status).toEqual('active');
				expect($rootScope.card.version).toEqual('');
				expect($rootScope.isVersionStateOkay()).toBe(false);
				$rootScope.card.version = 'fred';
				expect($rootScope.isVersionStateOkay()).toBe(true);
				$rootScope.card.status = 'inactive';
				$rootScope.card.version = '';
				expect($rootScope.isVersionStateOkay()).toBe(true);
			});

			it ('evaluates isCommentStateOkay correctly', function () {
				createController();
				$httpBackend.flush();
				//from active to active -> must have comment
				expect($rootScope.oldStatus).toEqual('active');
				expect($rootScope.card.status).toEqual('active');
				expect($rootScope.card.versionComment).toEqual('');
				expect($rootScope.isCommentStateOkay()).toBe(false);
				$rootScope.card.versionComment = 'fred';
				expect($rootScope.isCommentStateOkay()).toBe(true);

				//from active to inactive -> must have comment
				$rootScope.card.status = 'inactive';
				expect($rootScope.isCommentStateOkay()).toBe(true);
				$rootScope.card.versionComment = '';
				expect($rootScope.isCommentStateOkay()).toBe(false);

				//from inactive to active -> must have comment
				$rootScope.oldStatus = 'inactive';
				$rootScope.card.status = 'active';
				expect($rootScope.isCommentStateOkay()).toBe(false);
				$rootScope.card.versionComment = 'fred';
				expect($rootScope.isCommentStateOkay()).toBe(true);

				//from inactive to inactive -> comment optional
				$rootScope.card.status = 'inactive';
				expect($rootScope.isCommentStateOkay()).toBe(true);
				$rootScope.card.versionComment = '';
				expect($rootScope.isCommentStateOkay()).toBe(true);
			});

			afterEach(function() {
				$httpBackend.verifyNoOutstandingExpectation();
				$httpBackend.verifyNoOutstandingRequest();
			});
		});

		describe('cardmetametrics', function () {
			var ctrl,scope;

			beforeEach(function () {
				scope = {};
				ctrl = $controller('cardmetametrics', {$scope: scope});
			});

			it ('initializes dates and message object', function () {
				var now = new Date();
				expect(scope.fromdt.month).toEqual(now.getMonth()+1);
				expect(scope.fromdt.day).toEqual(1);
				expect(scope.fromdt.year).toEqual(now.getFullYear());

				expect(scope.todt.month).toEqual(now.getMonth()+1);
				expect(scope.todt.day).toEqual(now.getDate());
				expect(scope.todt.year).toEqual(now.getFullYear());

				checkMsgAllFalse(scope.msgFrom);
				checkMsgAllFalse(scope.msgTo);
				expect(scope.rangeOkay).toEqual(true);
				var day1floor = new Date(now.getFullYear(), now.getMonth(), 1, 0,0,0);
				expect(scope.stringFrom).toEqual(day1floor.getTime().toString());
				var day1ceil = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 23, 59, 59);
				expect(scope.stringTo).toEqual(day1ceil.getTime().toString());
			});

			it ('calculates valid dates as strings', function () {
				scope.fromdt.month = '2';
				scope.fromdt.day = '28';
				scope.fromdt.year = '2015';

				scope.todt.month = '2';
				scope.todt.day = '29';
				scope.todt.year = '2015';

				scope.update();

				checkMsgAllFalse(scope.msgFrom);
				expect(scope.msgTo.badDay).toEqual(true);
				expect(scope.msgTo.badMonth).toEqual(false);
				expect(scope.msgTo.badYear).toEqual(false);
				expect(scope.msgTo.badDate).toEqual(true);
				expect(scope.rangeOkay).toEqual(false);
				var day1floor = new Date(2015, 1, 28, 0,0,0);
				expect(scope.stringFrom).toEqual(day1floor.getTime().toString());
				expect(scope.stringTo).toEqual('0');
			});

			it ('calculates \'rangeOkay\' correctly', function () {
				scope.fromdt.month = '1';
				scope.fromdt.day = '1';
				scope.fromdt.year = '2015';

				scope.todt.month = '1';
				scope.todt.day = '31';
				scope.todt.year = '2015';

				scope.update();
				checkMsgAllFalse(scope.msgFrom);
				checkMsgAllFalse(scope.msgTo);
				expect(scope.rangeOkay).toEqual(true);
				var day1floor = new Date(2015, 0, 1, 0,0,0);
				var day2ceil = new Date(2015, 0, 31, 23, 59, 59);
				expect(scope.stringFrom).toEqual(day1floor.getTime().toString());
				expect(scope.stringTo).toEqual(day2ceil.getTime().toString());

				scope.fromdt.month = 2;

				scope.update();
				checkMsgAllFalse(scope.msgFrom);
				checkMsgAllFalse(scope.msgTo);
				expect(scope.rangeOkay).toEqual(false);
				day1floor = new Date(2015, 1, 1, 0,0,0);
				day2ceil = new Date(2015, 0, 31, 23, 59, 59);
				expect(scope.stringFrom).toEqual(day1floor.getTime().toString());
				expect(scope.stringTo).toEqual(day2ceil.getTime().toString());
			});

			it ('calculates month ends okay', function () {
				scope.fromdt.month = '1';
				scope.fromdt.day = '1';
				scope.fromdt.year = '2015';

				scope.todt.month = '1';
				scope.todt.day = '31';
				scope.todt.year = '2015';
				scope.update();
				checkMsgAllFalse(scope.msgFrom);
				checkMsgAllFalse(scope.msgTo);

				var shortMonths = [2,4,6,9,11];
				for (var i = shortMonths.length - 1; i >= 0; i--) {
					scope.todt.month = shortMonths[i]
					scope.update();
					expect(scope.msgTo.badDay).toEqual(true);
					expect(scope.msgTo.badMonth).toEqual(false);
					expect(scope.msgTo.badYear).toEqual(false);
				};
			});
		});
	});

});

describe ('clientserverCommon tests', function () {
	describe('clientserverCommon', function () {
		describe('nameRegex', function () {
			it ('validates names', function () {
				expect(clientserverCommon).toBeDefined();
				expect(clientserverCommon.nameRegex).toBeDefined();
				var aString = "i'll ";
				var result = aString.match(clientserverCommon.nameRegex);
				expect(result).toBeDefined();
				result = clientserverCommon.nameRegex.exec(aString);
				expect(clientserverCommon.nameRegex.exec('Jones\'-Henry Jr.')).toBeDefined();
			});

			it ('invalidates names', function () {
				var result = clientserverCommon.nameRegex.exec('hi there`');
				expect(result).toBeNull();
			});
		});

		describe('sysidRegex', function () {
			it ('validates sysids', function () {
				expect(clientserverCommon.sysidRegex.test('vhaislmonsos')).toBe(true);
				expect(clientserverCommon.sysidRegex.test('vhaislmonso_2')).toBe(true);
			});
		});

		describe('pad2', function () {
			it ('handles null entries', function () {
				expect(clientserverCommon.pad2()).toEqual('00');
			});

			it ('handles single digits', function () {
				expect(clientserverCommon.pad2(0)).toEqual('00');
				expect(clientserverCommon.pad2(9)).toEqual('09');
			});

			it ('handles double digits', function () {
				expect(clientserverCommon.pad2(10)).toEqual('10');
				expect(clientserverCommon.pad2(99)).toEqual('99');
			});
		});

		describe('endsWith()', function () {
			it ('handles null strings', function () {
				expect(clientserverCommon.endsWith()).toEqual(false);
				var a1,a2;
				expect(clientserverCommon.endsWith(a1,a2)).toEqual(false);
			});
			
			it ('handles null test string', function () {
				expect(clientserverCommon.endsWith(null, 'wow')).toEqual(false);
				var a1,a2 = 'fred';
				expect(clientserverCommon.endsWith(a1,a2)).toEqual(false);
			});

			it ('handles null end string', function () {
				expect(clientserverCommon.endsWith('wow', null)).toEqual(false);
				var a2,a1 = 'fred';
				expect(clientserverCommon.endsWith(a1,a2)).toEqual(false);
			});
			
			it ('handles false ending', function () {
				expect(clientserverCommon.endsWith('fred.woww', '.wow')).toEqual(false);
			});
			
			it ('matches properly', function () {
				expect(clientserverCommon.endsWith('file.html', '.html')).toEqual(true);
				expect(clientserverCommon.endsWith('file.html', '.htm')).toEqual(false);
			});
		});

	});
});

function checkMsgAllFalse (msg) {
	expect(msg.badDay).toEqual(false);
	expect(msg.badMonth).toEqual(false);
	expect(msg.badYear).toEqual(false);
	expect(msg.badDate).toEqual(false);
}