/* global Buffer */
/* global afterEach */
/* global beforeEach */
/* global describe */
/* global it */
//ui_helper_test.js
var should = require('should');
var helper = require('../routes/ui_helper');
var log4js = require('log4js');
var helperLog = log4js.getLogger('ui_helper');
var fs = require('fs');
var log = log4js.getLogger('ui_helper_test');
helperLog.setLevel('ERROR');

describe('ui_help.js tests', function () {
	describe('getCard()', function () {
		var req = null;

		beforeEach(function () {
			req = getMockReq1();
		});

		it ('handles empty tags', function () {
			var card = helper.getCard(req);
			card.tags.length.should.equal(0);
			should(Array.isArray(card.tags)).be.ok;
		});

		it ('handles one tag', function () {
			req.body.tags = ' hi ';
			var card = helper.getCard(req);
			card.tags.length.should.equal(1);
			should(Array.isArray(card.tags)).be.ok;
			card.tags[0].should.equal('hi');
		});

		it ('handles multiple tags with CR and LF', function () {
			req.body.tags = ' one \r\ntwo\n\rthr ';
			var card = helper.getCard(req);
			card.tags.length.should.equal(3);
			card.tags[0].should.equal('one');
			card.tags[1].should.equal('two');
			card.tags[2].should.equal('thr');
		});

		it ('handles files', function () {
			var card = helper.getCard(req);
			card.file.originalname.should.equal('fred.html');
		});

		it ('handles createdBy', function () {
			var card = helper.getCard(req);
			card.createdBy.should.equal('oit');
		});

		it ('handles status', function () {
			var card = helper.getCard(req);
			card.status.should.equal('active');
		});

		it ('handles replaceFile', function () {
			var card = helper.getCard(req);
			should(card.replaceFile).be.not.ok;

			card = helper.getCard(getMockReq2());
			card.replaceFile.should.equal(true);
		});

		it ('handles replaceOwner', function () {
			var card = helper.getCard(req);
			should(card.replaceOwner).be.not.ok;

			card = helper.getCard(getMockReq2());
			card.replaceOwner.should.be.ok;
			card.ownerId.should.equal(2233);
		});
	});

	describe('stringArrayToLines()', function () {
		it ('handles null array', function () {
			helper.stringArrayToLines().should.equal('');
		});

		it ('handles empty array', function () {
			helper.stringArrayToLines([]).should.equal('');
		});

		it ('handles string array', function () {
			helper.stringArrayToLines(['one', 'two', 'thr']).should.equal('one\r\ntwo\r\nthr');
		});
	});

	describe('getUser()', function () {
		it('happy path', function () {
			var req = {body:{lname: 'jones', fname:'fred', status: 'active', userType:'card', email:'somewhere@void.com'}};
			var user = helper.getUser(req);
			user.lastname.should.equal('jones');
			user.firstname.should.equal('fred');
			user.status.should.equal('active');
			user.userType.should.equal('card');
			user.email.should.equal('somewhere@void.com');
		});
	});

	describe('getDashDate()', function () {
		it ('happy path', function () {
			var dt = helper.getDashDate('1-1-2015');
			dt.getDate().should.equal(1);
			dt.getMonth().should.equal(0);
			dt.getFullYear().should.equal(2015);
		});

		it ('handles invalid or empty dates', function () {
			var dt = helper.getDashDate('fred');
			should(dt).be.not.ok;
			dt = helper.getDashDate('99-99-2014');
			should(dt).be.not.ok;
			dt = helper.getDashDate('');
			should(dt).be.not.ok;
			dt = helper.getDashDate();
			should(dt).be.not.ok;
		});
	});

	describe('writeMetricsWithIterator()', function () {
		var iter = null;
		var res = null;

		it ('handles write error on first write', function (done) {
			iter = new TestMetricsIterator();
			res = new Response(1);

			helper.writeMetricsWithIterator(res, iter, function (err) {
				iter.nextCallCount.should.equal(-1);
				iter.doneCallCount.should.equal(1);
				err.toString().should.equal(res.error.toString());
				res.writeList.length.should.equal(0);
				done();
			}, true);
		});

		it ('writes first line before first iter error', function (done) {
			iter = new TestMetricsIterator(1);
			res = new Response();
			helper.writeMetricsWithIterator(res, iter, function (err) {
				iter.nextCallCount.should.equal(0);
				iter.doneCallCount.should.equal(1);
				err.toString().should.equal('Error: ouchy');
				res.writeList.length.should.equal(1);
				res.writeList[0].should.equal('"timestamp","deviceHash","ip","request"');
				done();
			}, true);
		});

		it ('handles error on second write', function (done) {
			iter = new TestMetricsIterator();
			res = new Response(2);
			helper.writeMetricsWithIterator(res, iter, function (err) {
				iter.nextCallCount.should.equal(0);
				iter.doneCallCount.should.equal(1);
				err.toString().should.equal('Error: ouch');
				res.writeList.length.should.equal(1);
				res.writeList[0].should.equal('"timestamp","deviceHash","ip","request"');
				done();
			}, true);
		});

		it ('successful completion and calls \'done\'', function (done) {
			iter = new TestMetricsIterator();
			res = new Response();
			helper.writeMetricsWithIterator(res, iter, function (err) {
				iter.nextCallCount.should.equal(3);
				iter.doneCallCount.should.equal(1);
				res.writeList.length.should.equal(4);
				res.writeList[0].should.equal('"timestamp","deviceHash","ip","request"');
				res.writeList[1].should.equal('\r\n"2015-04-01 08:00:00","abcdefghijk","127.0.0.1","/pocketcardv1/user/abcdefghijk/cards"');
				res.writeList[2].should.equal('\r\n"2015-04-01 09:00:00","abcdefghijk","127.0.0.1","/pocketcardv1/user/abcdefghijk/cards?state=active"');
				res.writeList[3].should.equal('\r\n"2015-04-01 10:00:00","abcdefghijk","127.0.0.1","/pocketcardv1/user/abcxyz112233/cards/10101"');
				done();
			}, true);
		});
	});

	describe('writeMetricsWithIterator2()', function () {
		var iter = null;
		var res = null;

		it ('handles write error on first write', function (done) {
			iter = new TestMetricsIterator();
			res = new ResponseSync(1);

			helper.writeMetricsWithIterator2(res, iter, function (err) {
				iter.nextCallCount.should.equal(-1);
				iter.doneCallCount.should.equal(1);
				err.toString().should.equal(res.error.toString());
				res.writeList.length.should.equal(0);
				done();
			}, true);
		});

		it ('writes first line before first iter error', function (done) {
			iter = new TestMetricsIterator(1);
			res = new ResponseSync();
			helper.writeMetricsWithIterator2(res, iter, function (err) {
				iter.nextCallCount.should.equal(0);
				iter.doneCallCount.should.equal(1);
				err.toString().should.equal('Error: ouchy');
				res.writeList.length.should.equal(1);
				res.writeList[0].should.equal('"timestamp","deviceHash","ip","request"');
				done();
			}, true);
		});

		it ('handles error on second write', function (done) {
			iter = new TestMetricsIterator();
			res = new ResponseSync(2);
			helper.writeMetricsWithIterator2(res, iter, function (err) {
				iter.nextCallCount.should.equal(0);
				iter.doneCallCount.should.equal(1);
				err.toString().should.equal('Error: ouch2');
				res.writeList.length.should.equal(1);
				res.writeList[0].should.equal('"timestamp","deviceHash","ip","request"');
				done();
			}, true);
		});

		it ('successful completion and calls \'done\'', function (done) {
			iter = new TestMetricsIterator();
			res = new ResponseSync();
			helper.writeMetricsWithIterator2(res, iter, function (err) {
				iter.nextCallCount.should.equal(3);
				iter.doneCallCount.should.equal(1);
				res.writeList.length.should.equal(4);
				res.writeList[0].should.equal('"timestamp","deviceHash","ip","request"');
				res.writeList[1].should.equal('\r\n"2015-04-01 08:00:00","abcdefghijk","127.0.0.1","/pocketcardv1/user/abcdefghijk/cards"');
				res.writeList[2].should.equal('\r\n"2015-04-01 09:00:00","abcdefghijk","127.0.0.1","/pocketcardv1/user/abcdefghijk/cards?state=active"');
				res.writeList[3].should.equal('\r\n"2015-04-01 10:00:00","abcdefghijk","127.0.0.1","/pocketcardv1/user/abcxyz112233/cards/10101"');
				done();
			}, true);
		});
	});

	describe('getTimeDate()', function () {
		it ('returns null for any non number input', function () {
			should(helper.getTimeDate()).be.not.ok;
			should(helper.getTimeDate(null)).be.not.ok;
			should(helper.getTimeDate('')).be.not.ok;
			should(helper.getTimeDate(0)).be.not.ok;
			should(helper.getTimeDate('fred')).be.not.ok;
		});

		it ('returns a date for a given time entry', function () {
			var dt = helper.getTimeDate(100000000000);
			dt.getFullYear().should.equal(1973);
			dt = helper.getTimeDate('00012736826784');
			dt.getFullYear().should.equal(1970);
		});
	});

	describe('getAndDeleteFile', function () {
		var path = 'uploads/test.html';
		var dataString = '<html></html>';

		beforeEach(function (done) {
			if (!fs.existsSync('uploads')){
				fs.mkdirSync('uploads');
			}
			fs.writeFile(path, dataString, function (err) {
				if (err) throw err;
				done();
			});
		});

		it ('returns file and deletes if exists', function (done) {
			helper.getAndDeleteFile(path, function (err, data) {
				should(err).be.not.ok;
				data.toString().should.equal(new Buffer(dataString).toString());
				done();
			});
		});

		it ('returns an error if file doesn\'t exist', function (done) {
			helper.getAndDeleteFile('fred.htm', function (err, data) {
				should(err.toString().indexOf("fred.htm")>0).be.ok;
				should(data).be.not.ok;
				done();
			});
		});

		afterEach(function (done) {
			fs.unlink(path, function (err) {
				done();
			});
		});
	});

	describe('getCard2()', function () {
		var path = 'uploads/test.html';
		var dataString = '<html></html>';

		beforeEach(function (done) {
			if (!fs.existsSync('uploads')){
				fs.mkdirSync('uploads');
			}
			fs.writeFile(path, dataString, function (err) {
				if (err) throw err;
				done();
			});
		});

		it ('happy path all', function (done) {
			var fields = { 
				title: [ 'Test card 5-15-2015' ],
				descr: [ 'Test info card' ],
				tags: [ 'test\r\ninfo \r\ncard' ],
				createdBy: [ 'OIT testing' ],
				version: [ '1.0.0' ],
				versionComment: [ 'Initial' ],
				status: [ 'active' ],
				replaceFile: ['on'],
				replaceOwner: ['on'],
				ownerId: ['2']
			};
			var files = {
				file1: [{
							fieldName: 'file1',
							originalFilename: 'va1.htm',
							path: 'uploads/test.html',
							headers: {},
							size: 13
						}]
			};
			helper.getCard2(fields, files, function (err, card) {
				should(err).be.not.ok;
				card.title.should.equal('Test card 5-15-2015');
				card.descr.should.equal('Test info card');
				card.tags.length.should.equal(3);
				card.tags[0].should.equal('test');
				card.tags[1].should.equal('info');
				card.tags[2].should.equal('card');
				card.createdBy.should.equal('OIT testing');
				card.version.should.equal('1.0.0');
				card.versionComment.should.equal('Initial');
				card.status.should.equal('active');
				card.file.originalname.should.equal('va1.htm')
				card.file.path.should.equal('uploads/test.html');
				card.file.buffer.toString().should.equal('<html></html>');
				card.replaceOwner.should.be.ok;
				card.replaceFile.should.be.ok;
				card.ownerId.should.equal('2');
				done();
			});
		});

		afterEach(function (done) {
			fs.unlink(path, function (err) {
				done();
			});
		});
	});

	describe('getMonthYear()', function () {
		it ('returns string of zero length when null date', function () {
			helper.getMonthYear().should.equal('');
		});

		it ('returns toString if object is not a date', function () {
			helper.getMonthYear("fred").should.equal('fred');
		});

		it ('returns proper month and year', function () {
			helper.getMonthYear(new Date('5/21/2015')).should.equal('May 2015');
		});
	});

	describe('getSessionState()', function () {
		it ('throws error if request empty', function () {
			try {
				helper.getSessionState();
				throw new Error('not caught');
			}
			catch (err) {
				err.toString().should.equal('Error: request was empty');
			}
		});
		
		it ('throws error if request null', function () {
			try {
				helper.getSessionState(null);
				throw new Error('not caught');
			}
			catch (err) {
				err.toString().should.equal('Error: request was empty');
			}
		});
		
		it ('throws error if session empty', function () {
			try {
				helper.getSessionState({});
				throw new Error('not caught');
			}
			catch (err) {
				err.toString().should.equal('Error: session was empty');
			}
		});
		
		it ('creates empty stateObject if not found', function () {
			var req = {
				session: {}
			};
			var result = helper.getSessionState(req);
			should(result).be.ok;
		});
		
		it ('returns null if req is null', function () {
			should(helper.getSessionState(null, true)).be.not.ok;
		});
		
		it ('returns null if session is null', function () {
			should(helper.getSessionState({}, true)).be.not.ok;
		});
	});
	
	describe ('getExitUrl()', function () {
		it ('returns null if no matching environment variable and not node production', function () {
			should(helper.getExitUrl()).be.not.ok;
			should(helper.getExitUrl(null, null)).be.not.ok;
			should(helper.getExitUrl('fred', null)).be.not.ok;
		});
		
		it ('returns production link if no matching environment varaible and is node production', function () {
			var PROD_URL = 'https://ssologon.iam.domain/CentralLogin/centrallanding.aspx';
			helper.getExitUrl(null, 'production').should.equal(PROD_URL);
			helper.getExitUrl('fred', 'production').should.equal(PROD_URL);
		});
		
		it ('returns dev link if dev env variable passed in', function () {
			var DEV_URL = 'https://logon.dev2.iam.domain/CentralLogin/centrallanding.aspx';
			helper.getExitUrl('dev').should.equal(DEV_URL);
			helper.getExitUrl('dev', 'production').should.equal(DEV_URL);
		});
		
		it ('returns sqa link if sqa env variable passed in', function () {
			var SQA_URL = 'https://ssologon.sqa.iam.domain/CentralLogin/centrallanding.aspx';
			helper.getExitUrl('sqa').should.equal(SQA_URL);
			helper.getExitUrl('sqa', 'production').should.equal(SQA_URL);
		});
		
		it ('returns preprod link if preprod env variable passed in', function () {
			var PREPROD_URL = 'https://ssologon.preprod.iam.domain/CentralLogin/centrallanding.aspx';
			helper.getExitUrl('preprod').should.equal(PREPROD_URL);
			helper.getExitUrl('preprod', 'production').should.equal(PREPROD_URL);
		});
	});
	
	describe ('getLogoutUrl()', function () {
		it ('returns null if no matching environment variable and not node production', function () {
			should(helper.getLogoutUrl()).be.not.ok;
			should(helper.getLogoutUrl(null, null)).be.not.ok;
			should(helper.getLogoutUrl('fred', null)).be.not.ok;
		});
		
		it ('returns production link if no matching environment varaible and is node production', function () {
			var PROD_URL = 'https://ssologon.iam.domain/CentralLogin/idletimeout.aspx';
			helper.getLogoutUrl(null, 'production').should.equal(PROD_URL);
			helper.getLogoutUrl('fred', 'production').should.equal(PROD_URL);
		});
	});
});

function Response(callToThrowErrorOn) {
	if (!callToThrowErrorOn) {
		callToThrowErrorOn = 0;
	}
	this.writeList = [];
	this.errorCall = callToThrowErrorOn;
	this.error = null;
	this.write = function (aString, format, callback) {
		if (this.errorCall===this.writeList.length+1) {
			this.error = new Error('ouch');
			return callback(this.error);
		}
		this.writeList.push(aString);
		callback();
	};
}

function ResponseSync(callToThrowErrorOn) {
	if (!callToThrowErrorOn) {
		callToThrowErrorOn = 0;
	}
	this.writeList = [];
	this.errorCall = callToThrowErrorOn;
	this.error = null;
	this.write = function (aString, format) {
		if (this.errorCall===this.writeList.length+1) {
			this.error = new Error('ouch2');
			throw this.error;
		}
		this.writeList.push(aString);
	};
	return this;
}

function TestMetricsIterator(callToThrowErrorOn) {
	if (!callToThrowErrorOn) {
		callToThrowErrorOn = 0;
	}
	this.doneCallCount = 0;
	this.nextCallCount = -1;
	this.errorCall = callToThrowErrorOn;
	this.list = [
		{deviceHash : "abcdefghijk", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcdefghijk/cards", timestamp : new Date("2015-04-01 08:00:00")},
		{deviceHash : "abcdefghijk", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcdefghijk/cards?state=active", timestamp : new Date("2015-04-01 09:00:00")},
		{deviceHash : "abcdefghijk", clientIp : "127.0.0.1", request : "/pocketcardv1/user/abcxyz112233/cards/10101", timestamp : new Date("2015-04-01 10:00:00")}
	];
	this.getNext = function (callback) {
		this.nextCallCount++;
		if (this.errorCall===(this.nextCallCount+1)) {
			return callback(new Error('ouchy'));
		}
		var result = null;
		if (this.nextCallCount<this.list.length) {
			result = this.list[this.nextCallCount];
		}
		callback(null, result);
	};
	this.done = function() {
		this.doneCallCount++;
	};
}

function getMockReq1() {
	var req = {}
	req.body = {};
	req.body.title = 'title';
	req.body.tags = '';
	req.body.version = 'version';
	req.body.status = 'active';
	req.body.versionComment = 'vc';
	req.files = {};
	req.files.file1 = {originalname: 'fred.html'};
	req.body.createdBy = "oit";
	return req;
}

function getMockReq2() {
	var req = getMockReq1();
	req.body.replaceFile = true;
	req.body.replaceOwner = true;
	req.body.ownerId = 2233;
	return req;
}
