//utils_test.js
var should = require('should');
var utils = require('../../common/utils');

describe('utils.js tests', function () {
	describe('getMerged()', function () {
		it ('handles 2 empty objects', function () {
			should(utils.getMerged()).be.not.ok;
		});

		var testObject = getTestObject1();

		it ('copies 1 non-null from right side', function () {
			var obj = utils.getMerged(null, testObject);
			testObject.should.not.be.exactly(obj);
			testObject.should.match(obj);
		});

		it ('copies 1 non-null from left side', function () {
			var obj = utils.getMerged(testObject, null);
			testObject.should.not.be.exactly(obj);
			testObject.should.match(obj);
		});

		var testObject2 = getTestObject2();

		it ('happy path', function () {
			var obj = utils.getMerged(testObject, testObject2);
			obj.wow.should.equal('weird');
			obj.adate.should.equal(testObject.adate);
			obj.bdate.should.equal(testObject2.bdate);
		});
	});

	describe('getMergedInto()', function () {
		it ('handles 2 empty objects', function () {
			should(utils.getMergedInto()).be.not.ok;
		});

		it ('creates obj1 if null', function () {
			var to1 = getTestObject1();
			var obj = utils.getMergedInto(null, to1);
			obj.should.not.be.exactly(to1);
			obj.adate.should.equal(to1.adate);
			obj.wow.should.equal(to1.wow);
		});

		it ('happy path', function () {
			var to1 = getTestObject1();
			var to2 = getTestObject2();
			var obj = utils.getMergedInto(to1, to2);
			obj.should.be.exactly(to1);
			to1.wow.should.equal('weird');
			to1.adate.should.equal(obj.adate);
			to1.bdate.should.equal(to2.bdate);
		});
	});
});

function getTestObject1 () {
	return {wow:'cool', adate: new Date('2015-02-01 00:00:00')};
}

function getTestObject2() {
	return {bdate: new Date('2015-02-02 22:22:22'), wow: 'weird'};
}