/* global afterEach */
/* global beforeEach */
/* global it */
/* global describe */
//Mongo_dao_itest_test.js
var log4js = require('log4js');
var should = require('should');
var Dao = require('../dao/Mongo_dao');
var _ = require('underscore');
var mclient = require('mongodb').MongoClient;
log4js.getLogger('Mongo_dao').setLevel('FATAL');

var log = log4js.getLogger('Mongo_dao_itest_test.js');
log.setLevel('FATAL');

var validConfig = {
	url: '',
	dbName: '',
	//url: 'mongodb://localhost:27017',
	//dbName: '/pc2',
	getConnectionString: function () {
		return this.url + this.dbName;
	}
}

var invalidConfig = {
	url: 'mongodb://localhost:11111',
	dbName: '/pcpc'
}

describe('Mongo_dao Integration Tests', function () {
	describe ('Invalid connection tests', function () {
		var dao = new Dao(invalidConfig, {
			connect: function (connectString, callback) {
				callback(new Error("failed to connect"));
			}
		});

		it ('getAllCurrentCards (by status)', function (done) {
			dao.getAllCurrentCards(function (err, results) {
				assertConnectionError(err);
				should(results).be.not.ok;
				done();
			});
		});

		it ('getAllCurrentCards (by IDs)', function (done) {
			dao.getAllCurrentCards({ids:[1]}, function (err, results) {
				assertConnectionError(err);
				should(results).be.not.ok;
				done();
			});
		});

		it ('getCardContent', function (done) {
			dao.getCardContent(1, function (err, results) {
				assertConnectionError(err);
				should(results).be.not.ok;
				done();
			});
		});

		it ('saveMetrics', function (done) {
			dao.saveMetrics(1, 2, 3, 4, function (err, results) {
				assertConnectionError(err);
				should(results).be.not.ok;
				done();
			});
		});
		
		it ('getSites', function (done) {
			dao.getSites(function (err, results) {
				assertConnectionError(err);
				should(results).be.not.ok;
				done();
			});
		});
	});

	if (!validConfig.url || !validConfig.dbName) {
		log.fatal('*** Some Mongo_dao TEST NOT RUN ***');
		log.fatal('*** To run the missing tests, enter mongo instance connection information ***');
	}
	else {
		describe('getCardContent()', function () {

			it ('returns nothing for unfound cards', function (done) {
				var dao = new Dao(validConfig);
				dao.getCardContent(10101, function (err, result) {
					if (err) {
						throw err;
					}
					should(result).be.not.ok;
					done();
				});
			});

			it ('returns content of a card (with a nummber)', function (done) {
				var dao = new Dao(validConfig);
				dao.getCardContent(2, function (err, result) {
					if (err) {
						throw err;
					}
					//log.debug('===card content===');
					//log.debug(result);
					should(result).be.ok;
					done();
				});
			});

			it ('returns content of a card (with a number as a string)', function (done) {
				var dao = new Dao(validConfig);
				dao.getCardContent('2', function (err, result) {
					if (err) {
						throw err;
					}
					should(result).be.ok;
					done();
				});
			});

		});

		describe('getAllCurrentCards()', function () {
			
			describe('with IDs specified', function () {
				var dao = null;

				beforeEach(function () {
					dao = new Dao(validConfig);
				});

				it ('returns nothing when one search and one not found (without array)', function (done) {
					dao.getAllCurrentCards({ids:10101}, function (err, results) {
						results.length.should.equal(0);
						done();
					});
				});

				it ('returns nothing when one search and one not found (with array)', function (done) {
					dao.getAllCurrentCards({ids:10101}, function (err, results) {
						results.length.should.equal(0);
						done();
					});
				});

				it ('returns one when one searched found (with array)', function (done) {
					dao.getAllCurrentCards({ids:[1]}, function (err, results) {
						results.length.should.equal(1);
						results[0].id.should.equal(1);
						results[0].siteId.should.equal(99);
						done();
					});
				});

				it ('returns one when one searched found (without array)', function (done) {
					dao.getAllCurrentCards({ids:1}, function (err, results) {
						results.length.should.equal(1);
						results[0].id.should.equal(1);
						results[0].siteId.should.equal(99);
						done();
					});
				});

				it ('returns all when all found', function (done) {
					dao.getAllCurrentCards({ids:[1,2]}, function (err, results) {
						results.length.should.equal(2);
						if (results[0].id==1) {
							results[1].id.should.equal(2);
							results[1].siteId.should.equal(99);
						}
						else {
							results[1].id.should.equal(1);
							results[1].siteId.should.equal(99);
						}
						done();
					});
				});

				it ('returns one when one of many found', function (done) {
					dao.getAllCurrentCards({ids:[10101,1]}, function (err, results) {
						results.length.should.equal(1);
						results[0].id.should.equal(1);
						results[0].siteId.should.equal(99);
						done();
					});
				});

				it ('returns one specified with number as string', function (done) {
					dao.getAllCurrentCards({ids:'1'}, function (err, results) {
						results.length.should.equal(1);
						results[0].siteId.should.equal(99);
						done();
					});
				});
			});

			describe('with status specified', function () {
				var dao = null;
				beforeEach(function () {
					dao = new Dao(validConfig);
				});

				it ('returns active status cards', function (done) {
					dao.getAllCurrentCards({status:'active'}, function (err, results) {
						if (err) {
							throw err;
						}
						results.length.should.equal(1);
						results[0].id.should.equal(2);
						results[0].siteId.should.equal(99);
						done();
					});
				});

				it ('returns inactive status cards', function (done) {
					dao.getAllCurrentCards({status:'inactive'}, function (err, results) {
						if (err) {
							throw err;
						}
						results.length.should.equal(2);
						results.sort(function (i1, i2) {
							return i1.id - i2.id;
						});
						results[0].id.should.equal(1);
						results[1].id.should.equal(3);
						results[0].siteId.should.equal(99);
						results[1].siteId.should.equal(99);
						done();
					});
				});

				it ('returns empty for status not found', function (done) {
					dao.getAllCurrentCards({status:'fred'}, function (err, results) {
						if (err) {
							throw err;
						}
						results.length.should.equal(0);
						done();
					});
				});
			});

			describe('with nothing specified', function () {
				var dao = null;
				beforeEach(function () {
					dao = new Dao(validConfig);
				});

				it ('returns all current entries (empty parameters)', function (done) {
					dao.getAllCurrentCards({}, function (err, results) {
						results.length.should.equal(3);
						results.forEach(function (card) {
							card.siteId.should.equal(99);
						});
						done();
					});
				});

				it ('returns all current entries (no parameters)', function (done) {
					dao.getAllCurrentCards(function (err, results) {
						results.length.should.equal(3);
						results.forEach(function (card) {
							card.siteId.should.equal(99);
						});
						done();
					});
				});
			});
		});

		describe('saveMetrics()', function () {
			var hashKey = 'abcdefghij';
			it ('saves data', function (done) {
				var dao = new Dao(validConfig);
				var date = new Date();
				var req = 'fred/jane/whatever';
				var clientIp = '127.0.0.1';
				dao.saveMetrics(hashKey, clientIp, req, date, function (err) {
					mclient.connect(validConfig.getConnectionString(), function (err, db) {
						db.collection('metrics').find({deviceHash: hashKey}).toArray(function (err, results) {
							db.close();
							should(results.length>0).be.ok;
							results[0].deviceHash.should.equal(hashKey);
							results[0].request.should.equal(req)
							results[0].clientIp.should.equal(clientIp);
							results[0].timestamp.toString().should.equal(date.toString());
							done();
						});
					});
				});
			});

			afterEach(function (done) {
				mclient.connect(validConfig.getConnectionString(), function (err, db) {
					if (err) {
						throw err;
					}
					db.collection('metrics').remove({deviceHash: hashKey}, function (err, results) {
						db.close();
						if (err) {
							throw err;
						}
						done();
					});
				});
			});
		});
		
		describe('getSites()', function () {
			var dao;
			
			beforeEach(function (done) {
				dao = new Dao(validConfig);
				setupBogusSites(function (err) {
					if (err) throw err;
					done();
				});
			});
			
			it ('gets current active sites', function (done) {
				dao.getSites(function (err, sites) {
					if (err) throw err;
					sites = _.filter(sites, function (site) {
						return site.id>=99990;
					});
					sites.length.should.equal(6);
					_.where(sites , {id:99992})[0].descr.should.equal('two 1');
					_.where(sites , {id:99993})[0].descr.should.equal('three 2');
					_.where(sites , {id:99994})[0].descr.should.equal('four 1');
					_.where(sites , {id:99995})[0].descr.should.equal('five 1');
					_.where(sites , {id:99996})[0].descr.should.equal('six 2');
					_.where(sites , {id:99997})[0].descr.should.equal('seven 3');
					done();
				});
			});
			
			afterEach(function (done) {
				removeBogusSites(function (err) {
					if (err) throw err;
					done();
				});
			});
		});
	}
});

function setupBogusSites(callback) {
	mclient.connect(validConfig.getConnectionString(), function (err, db) {
		if (err) return callback(err);
		db.collection("sites").insert(getTestSites(), function (err) {
			db.close();
			callback(err);
		});
	});
}

function removeBogusSites(callback) {
	mclient.connect(validConfig.getConnectionString(), function (err, db) {
		if (err) return callback(err);
		db.collection("sites").remove({$and: [{id: {$gte: 99990}}, {id: {$lte: 99997}}]}, function (err) {
			db.close();
			callback(err);
		});
	});
}

function getTestSites() {
	var dt = new Date();
	return [
		{id: 99990, name: "zero", descr: "zero 1", status: "inactive", effectiveDate: getDateMonthOffset(dt, -1)},
		
		{id: 99991, name: "one", descr: "one 1", status: "active", effectiveDate: getDateMonthOffset(dt, -2)},
		{id: 99991, name: "one", descr: "one 2", status: "inactive", effectiveDate: getDateMonthOffset(dt, -1)},
		
		{id: 99992, name: "two", descr: "two 1", status: "active", effectiveDate: getDateMonthOffset(dt, -1)},
		
		{id: 99993, name: "three", descr: "three 1", status: "inactive", effectiveDate: getDateMonthOffset(dt, -2)},
		{id: 99993, name: "three", descr: "three 2", status: "active", effectiveDate: getDateMonthOffset(dt, -1)},
		
		{id: 99994, name: "four", descr: "four 1", status: "active", effectiveDate: getDateMonthOffset(dt, -1)},
		{id: 99994, name: "four", descr: "four 2", status: "inactive", effectiveDate: getDateMonthOffset(dt, 1)},
		
		{id: 99995, name: "five", descr: "five 1", status: "active", effectiveDate: getDateMonthOffset(dt, -1)},
		{id: 99995, name: "five", descr: "five 2", status: "inactive", effectiveDate: getDateMonthOffset(dt, 1)},
		{id: 99995, name: "five", descr: "five 3", status: "inactive", effectiveDate: getDateMonthOffset(dt, 2)},
		
		{id: 99996, name: "six", descr: "six 1", status: "inactive", effectiveDate: getDateMonthOffset(dt, -2)},
		{id: 99996, name: "six", descr: "six 2", status: "active", effectiveDate: getDateMonthOffset(dt, -1)},
		{id: 99996, name: "six", descr: "six 3", status: "inactive", effectiveDate: getDateMonthOffset(dt, 1)},
		
		{id: 99997, name: "seven", descr: "seven 1", status: "inactive", effectiveDate: getDateMonthOffset(dt, -10)},
		{id: 99997, name: "seven", descr: "seven 2", status: "inactive", effectiveDate: getDateMonthOffset(dt, -9)},
		{id: 99997, name: "seven", descr: "seven 3", status: "active", effectiveDate: getDateMonthOffset(dt, -8)}
	];
}

function getDateMonthOffset(aDate, offset) {
	var newDate = new Date(aDate);
	newDate.setMonth(aDate.getMonth()+offset);
	return newDate;
}

function assertConnectionError(err) {
	if (err.message.indexOf('failed to connect')<0) {
		throw new Error('wrong message:' + err.message);
	}
}
