/* global it */
/* global describe */
//service_itest_test.js
var Client = require('node-rest-client').Client;
var log4js = require('log4js');
var should = require('should');

var log = log4js.getLogger('service_itest_test');
var rest = new Client();


//***************************************
// Change this value to run against a different server
//
var host = {
	//url: ''
	url: 'http://localhost:7010'
};

describe('service.js integration tests', function () {
	if (host && host.url) {
		describe('/pocketcardv1/user/:device_hash/cards', function () {
			it ('returns all data', function (done) {
				cardsTest('pocketcardv1', done);
			});

			it ('returns active cards', function (done) {
				cardsActiveTest('pocketcardv1', done);
			});

			it ('returns inactive cards', function (done) {
				cardsInactiveTest('pocketcardv1', done);
			});
		});

		describe('/pocketcardv2/user/:device_hash/cards', function () {
			it ('returns all data', function (done) {
				cardsTest('pocketcardv2', done);
			});

			it ('returns active cards', function (done) {
				cardsActiveTest('pocketcardv2', done);
			});

			it ('returns inactive cards', function (done) {
				cardsInactiveTest('pocketcardv2', done);
			});
		});

		describe('/pocketcardv1/user/:device_hash/cards/:id', function () {
			it ('returns empty for not found card', function (done) {
				cardsByIdEmptyTest('pocketcardv1', done);
			});

			it ('returns one card when found', function (done) {
				cardsById1FoundTest('pocketcardv1', done);
			});

			it ('returns 2 cards when both found', function (done) {
				cardsById2FoundTest('pocketcardv1', done);
			});
		});

		describe('/pocketcardv2/user/:device_hash/cards/:id', function () {
			it ('returns empty for not found card', function (done) {
				cardsByIdEmptyTest('pocketcardv2', done);
			});

			it ('returns one card when found', function (done) {
				cardsById1FoundTest('pocketcardv2', done);
			});

			it ('returns 2 cards when both found', function (done) {
				cardsById2FoundTest('pocketcardv2', done);
			});
		});

		describe('/pocketcardv1/user/:device_hash/cards/:id/contents', function () {
			it ('returns empty if not valid', function (done) {
				cardContentEmptyTest('pocketcardv1', done);
			});

			it ('returns contents', function (done) {
				cardContentFoundTest('pocketcardv1', done);
			});
		});

		describe('/pocketcardv2/user/:device_hash/cards/:id/contents', function () {
			it ('returns empty if not valid', function (done) {
				cardContentEmptyTest('pocketcardv2', done);
			});

			it ('returns contents', function (done) {
				cardContentFoundTest('pocketcardv2', done);
			});
		});
		
		describe ('/pocketcardv2/user/:device_hash/sites', function () {
			it ('returns active sites', function (done) {
				rest.get(host.url + '/pocketcardv2/user/abcxyz112233/sites', function (data, response) {
					if (response.complete) {
						data.length.should.equal(1);
						data[0].status.should.equal('active');
						data[0].id.should.equal(99);
						data[0].name.should.equal('V.A. National');
						data[0].descr.should.equal('Pocket Cards for the Department of Veterans Affairs');
						(typeof data[0].effectiveDate).should.equal('string');
						done();
					}
					else {
						throw new Error('incomplete');
					}
				});
			});
		});

	}
	else {
		console.log('~*~*~*~*~*~ TESTS NOT RUN on service_itest_test.js! ~*~*~*~*~*~');
		console.log('~* If you want these tests to run, then');
		console.log('~* set the host.url in this file to a');
		console.log('~* valid endpoint.');
	}
});

function cardsTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/abcdefghijk/cards', function (data, response) {
		if (response.complete) {
			var list = JSON.parse(data);
			list.length.should.equal(1);
			var card = list[0];
			card.id.should.equal(2);
			card.siteId.should.equal(99);
		}
		else {
			throw new Error('no data');
		}
		doneCallback();
	});
}

function cardsActiveTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/abcdefghijk/cards?state=active', function (data, response) {
		if (response.complete) {
			var list = JSON.parse(data);
			list.length.should.equal(1);
			var card = list[0];
			card.id.should.equal(2);
			card.siteId.should.equal(99);
		}
		else {
			throw new Error('incomplete');
		}
		doneCallback();
	});
}

function cardsInactiveTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/abcdefghijk/cards?state=inactive', function (data, response) {
		if (response.complete) {
			var list = JSON.parse(data);
			list.length.should.equal(2);
			list[0].siteId.should.equal(99);
			list[1].siteId.should.equal(99);
		}
		else {
			throw new Error('incomplete');
		}
		doneCallback();
	});
}

function cardsByIdEmptyTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/abcxyz112233/cards/10101', function (data, response) {
		if (response.complete) {
			data.toString().should.equal('[]');
			doneCallback();
		}
		else {
			throw new Error('incomplete');
		}
	});
}

function cardsById1FoundTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/abcxyz112233/cards/1', function (data, response) {
		if (response.complete) {
			var list = JSON.parse(data);
			list.length.should.equal(1);
			list[0].status.should.equal('inactive');
			list[0].siteId.should.equal(99);
			doneCallback();
		}
		else {
			throw new Error('incomplete');
		}
	});
}

function cardsById2FoundTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/abcxyz112233/cards/2,3', function (data, response) {
		if (response.complete) {
			var list = JSON.parse(data);
			list.length.should.equal(2);
			var card1 = null;
			var card2 = null;
			if ('active'==list[0].status) {
				card1 = list[0];
				card2 = list[1];
			}
			else {
				card2 = list[0];
				card1 = list[1];
			}
			card1.id.should.equal(2);
			card2.id.should.equal(3);
			card1.siteId.should.equal(99);
			card2.siteId.should.equal(99);
			doneCallback();
		}
		else {
			throw new Error('incomplete');
		}
	});
}

function cardContentEmptyTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/qwertyking/cards/10101/contents', function (data, response) {
		if (response.complete) {
			data.toString().should.equal('');
			doneCallback();
		}
		else {
			throw new Error('incomplete');
		}
	});
}

function cardContentFoundTest (serviceId, doneCallback) {
	rest.get(host.url + '/' + serviceId + '/user/qwertyking/cards/2/contents', function (data, response) {
		if (response.complete) {
			should(data).be.ok;
			doneCallback();
		}
		else {
			throw new Error('incomplete');
		}
	});
}
